'From Squeak3.7gamma of ''17 July 2004'' [latest update: #5987] on 23 November 2004 at 10:10:21 pm'!
"Change Set:		TimeZoneDatabase-V1-2-3-dtl
Date:			23 November 2004
Author:			David T. Lewis

Better integration of TimeZoneDatabase with Squeak date and time. Squeak
will now accurately track the time zone offset across daylight savings time
(DST) transitions and leap second transitions. Date and time arithmetic does
the right thing if a duration includes as DST transition or leap second
adjustment.

These changes Squeak are to be applied to the base TimeZoneDatabase package
(circa 1999).

Requires #asScaledDecimal and other patches.

Numeric format of PointInTime is changed from Float to ScaledDecimal.

Squeak DateAndTime is correctly reported across DST transitions and leap
second transitions.

TimeZoneProxy provides a TimeZone that can be set to any time zone
in the TimeZoneDatabase. DateAndTime uses the current setting of the
time zone database, and any change to the default time zone in the
database is automatically reflected in DateAndTime (as well as Date,
Time, and Duration).

Time zone offset is automatically updated at DST transitions.

Date and time arithmetic accounts for DST and leap second changes.

Maintains backward compatibility with Squeak Smalltalk seconds
clock, which reports local time seconds count without time zone
offset, daylight savings time, and leap second changes.

Deduces correct DateAndTime from the local Squeak seconds clock
for the time period before and after a 'fall back' DST transition
(uses TimeZoneDstTransitionWatcher to remember recent DST transitions).
The time zone must be set to a time zone that matches that of the
underlying operating system in order for this to work properly.

Maintains distinctions between PointInTime (atomic clock seconds),
DateAndTime (rotational time with Posix seconds), and Squeak (local
wall clock seconds) for conversions and time arithmetic.

TimePlugin has been updated to work with current VMMaker.

"!

Magnitude subclass: #PointInTime
	instanceVariableNames: 'absoluteTime'
	classVariableNames: 'NumericRepresentationSelector Platform'
	poolDictionaries: ''
	category: 'Time-UTC'!

!PointInTime commentStamp: 'dtl 10/23/2004 10:28' prior: 0!
I represent a single point on an infinite time line. My origin is arbitrarily
chosen to be the first instant of the day of January 1, 1970, UTC. My unit
of measure is the second, which I may represent as a Float, a Double, or a
ScaledDecimal. My class method #numericRepresentationClass: controls the
actual data type of the numeric resolution. This is set during class initialization
but can be changed at any time.

I am based on the naive assumptions that time can be represented as a single
continuum which may be experienced identically at different physical locations
and velocities, and such that two observers in two different locations can identify
a single PointInTime as a simultaneous event. These assumptions are not valid
when relativity is taken into account, but are acceptable for use in Newtonian
physics and Smalltalk.

A Float or Double has sufficient precision to represent a millisecond clock for
thousands of years, and can represent a microsecond clock for about the next
hundred years. This representation of time is used in order to emphasize the notion
of time as an infinite continuum, rather than a discreet integral value as implied
by computer hardware clocks. In other words, if there is to be a particle nature
to time, it should derive from physics and not from the peculiarities of computer
hardware.

In the event that a microsecond clock is required for Squeak in the
twenty-second century or later, the precision can be increased by shifting
the origin of time forward a couple hundred years (and adjusting conversion
methods accordingly), by changing to a higher precision numeric represention
(LargePositiveInteger or higher precision floating point data type), or by using
ScaledDecimal rather than a floating point representation.

Platform specific notes:

Squeak and Smalltalk/X use double precision floating point for class Float.
VisualWorks uses single precision float for Float, and double precision for
class Double. Therefore, the magnitude of a PointInTime is represented by
a Float or a Double, whichever provides the higher resolution.

On Squeak, access to the underlying high precision clock may be provided
by a pluggable primitive. See the methods in category primitives for examples,
and see the optional class TimePlugin for an implementation of the plugin.
Squeak has no built in concept of time zones or location-independent time
represention.

Smalltalk/X has an excellent implementation of time zones and time in
class AbsoluteTime. This should be used for any direct access to the
underlying system clock. For most practical applications, it also provides
excellent support for time zones using the underlying operating system
services.

VisualWorks provides access to the underlying operating system clock in
class Time, and some support for time zones in class TimeZone. The
implementation of TimeZone is portable across operating systems, but
is rather simplistic compared to Smalltalk/X.

Class PointInTime may be used in conjunction with a TimeZoneDatabase
to provide local time representation at any time and in any location
for which a TimeZoneRuleSet (time zone data set) is available. This
permits a platform independent implementation, but requires loading
time zone information into the Smalltalk image rather than accessing
it through operating system services.
!

TestCase subclass: #TimeTransformTest
	instanceVariableNames: 'saveLocalTimeZone saveLocalTransform saveLastTransitionTime dstTransitionAbsoluteSeconds dstTransitionPosixSeconds leapSeconds'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Time-TimeZones-tests'!

!TimeTransformTest commentStamp: 'dtl 11/14/2004 18:33' prior: 0!
Unit tests for local time transforms (time zone rules), and for integration of time zone rules with date and time classes. Two local time transforms are used, one that makes use of the leap second table (right/America/Detroit), and another that ignores the leap second table (America/Detroit). The tests check for correct time zone offsets before and after the Fall 2004 Daylight Savings transition for Detroit, Michigan USA, and for times before and after a leap second transition. A system time zone database with the necessary time zones must exist prior to running the tests.

See method category "documentation" for an explanation of time representations.

Some of these tests will fail if the TimePlugin is being used. TimePlugin reports the operating system view of Posix seconds, while TimeZoneDatsbase calculates Posix seconds from local seconds using the time zone offset. The tests that are expected to file have method comments to that effect. With TimePlugin disabled, all tests should pass (but PointInTime will not have access to the millisecond clock provided by TimePlugin).

!

Object subclass: #TimeZoneDatabase
	instanceVariableNames: 'defaultLocation defaultTimeZone timeZones leapSecondRuleSet indexSeparator'
	classVariableNames: 'OsPathNameSeparator ThisSystemDatabase'
	poolDictionaries: ''
	category: 'Time-TimeZones'!
Object subclass: #TimeZoneDstTransitionWatcher
	instanceVariableNames: 'lastTick transitionTick monitorProcess accessProtect'
	classVariableNames: 'DstMonitor'
	poolDictionaries: ''
	category: 'Time-TimeZones'!

!TimeZoneDstTransitionWatcher commentStamp: 'dtl 10/31/2004 09:32' prior: 0!
Squeak uses a Time>>totalSeconds clock that jumps forward or backward at daylight savings time transitions. During the one hour period following a "fall back" transition, the calculation of DateAndTime>>now is ambiguous.

This class provides a monitor to keep track of any recent "fall back" transitions, enabling the correct DateAndTime to be chosen for any value of the Squeak seconds clock. The #fallBackOccurredDuringHourBefore: method provides a test to determine if the fall back transition has occurred withing the last hour.

The monitor process runs continuously, and therefore uses a certain amount of CPU that may be noticeable on small systems.

It is possible to provide a primitive for system time that answers both the current seconds clock and the current time zone offset. This primitive would make the time zone monitor unnecessary.!

Object subclass: #TimeZoneProxy
	instanceVariableNames: 'abbreviation transform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Time-TimeZones'!

!TimeZoneProxy commentStamp: 'dtl 11/14/2004 13:33' prior: 0!
TimeZoneProxy is a TimeZone that obtains its information from an instance of TimeZoneRuleSet. This may be the LocalTimeTransform for the local time zone on this system obtained from a TimeZoneDatabase; or it may be the TimeZoneDatabase itself, in which case the proxy refers to the currently active LocalTimeTransform for the database.

To set the system time zone to a proxy for the time zone database:
	TimeZoneProxy setDynamicTransform

To set the local time zone for Detroit using a time zone that understands leap seconds:
	TimeZoneDatabase setLocalTimeZone: 'right/America/Detroit'
Or for a Detroit time zone that ignores leap seconds:
	TimeZoneDatabase setLocalTimeZone: 'posix/America/Detroit'

When configured in this manner, the name of the current local time zone is:
	DateAndTime localTimeZone transform defaultTimeZone timeZoneName

The chain of references is as follows:
	DateAndTime class>>localTimeZone ==> aTimeZoneProxy
	 aTimeZoneProxy>>transform ==> aTimeZoneDatabase
	   aTimeZoneDatabase>>defaultTimeZone ==> aTimeZoneRuleSet

This configuration permits DateAndTime to use UTC offset, daylight savings adjustment, and leap second adjustment for any point in time. When a daylight savings time transition occurs or a leap second adjustment is made, dates and times will automatically be adjusted accordingly. If a different time zone is selected as the current time zone in the database, DateAndTime will automatically refer to the new setting.

To undo the above configuration and return DateAndTime to its default configuration:
	DateAndTime localTimeZone: TimeZone default

This default setting may be required for existing DateAndTime test cases to pass, because some of the tests are not aware of leap second adjustments.

Notes:
To find a suitable time zone for e.g. Detroit, use:
	TimeZoneDatabase grepFor: 'Detroit'
To set the a dynamic time zone proxy for a time zone in one step:
	TimeZoneProxy setDynamicTransform: 'posix/America/Detroit'
If you want to set the time zone abbreviation to a preferred value:
	TimeZoneProxy setDynamicTransform: 'posix/America/Detroit' abbreviation: 'EST'

!


!DateAndTime methodsFor: 'smalltalk-80' stamp: 'dtl 10/31/2004 14:15'!
asSeconds
	"Return the number of seconds since the Squeak epoch"

	^ self asPointInTime asLocalSmalltalkSeconds
! !

!DateAndTime methodsFor: '*time-UTC' stamp: 'dtl 11/10/2004 05:43'!
asPointInTime

	"DateAndTime now asPointInTime"

	^ self asPointInTimeInTimeZone: LocalTimeZone! !

!DateAndTime methodsFor: '*time-UTC' stamp: 'dtl 11/14/2004 23:01'!
asPointInTimeInTimeZone: aTimeZone
	"The time zone is used only for determining the leap second offset for
	rotational time versus absolute time. DateAndTime is rotational time (aligned
	to the rotation of the planet), and PointInTime is absolute time (atomic clock
	seconds). An iterative method is used to ensure an acceptable result at leap
	second transitions, for which two PointInTime seconds may correspond to a
	single DateAndTime second."

	| pt leap newTime |
	pt _ PointInTime fromAbsoluteSeconds: jdn - SqueakEpoch * SecondsInDay
		- PointInTime posixOffset
		+ seconds - offset asSeconds + (nanos / 1000000000).
	(1 to: 10)
		inject: pt
		into: [:time :i |
			leap _ aTimeZone leapSecondsAt: time.
			newTime _ PointInTime fromAbsoluteSeconds: pt absoluteTime + leap.
			(newTime = time)
				ifTrue: [^ time].
			newTime].
	self notify: 'algorithm does not converge'.
	"Lack of convergence may occur if there has been a negative leap second.
	As of October 2004, there had never been a negative leap second, so if you
	are reading this now, please check the method and figure out if it is correct.
	It should be OK to just answer the value of newTime."
	^ newTime! !


!DateAndTime class methodsFor: 'ansi protocol' stamp: 'dtl 10/31/2004 09:59'!
now
	^ PointInTime now asDateAndTime
! !

!DateAndTime class methodsFor: 'ansi protocol' stamp: 'dtl 11/23/2004 19:04'!
year: year month: month day: day hour: hour minute: minute second: second
	"Return a DateAndTime. Start by assuming the current local time zone offset,
	and search for the correct offset to use with this DateAndTime (which may
	differ as a result of a daylight savings time transition)."


	| dt newOffset |
	(1 to: 10)
		inject: self localOffset
		into: [:offset :i |
			dt _ self
				year: year
				month: month
				day: day
				hour: hour
				minute: minute
				second: second
				offset: offset.
			newOffset _ Duration seconds: (self localTimeZone localOffsetSecondsAt: dt asPointInTime).
			(offset = newOffset) ifTrue: [^ dt].
			newOffset].
	self error: 'algorithm does not converge on a DateAndTime'
! !

!DateAndTime class methodsFor: 'squeak protocol' stamp: 'dtl 11/13/2004 13:47'!
localTimeZone: aTimeZone
	"Set the local time zone"

	"
	DateAndTime localTimeZone: (TimeZone offset:  0 hours name: 'Universal Time' abbreviation: 'UTC').
	DateAndTime localTimeZone: (TimeZone offset: -8 hours name: 'Pacific Standard Time' abbreviation: 'PST').
	"

	LocalTimeZone := aTimeZone.
	self changed: #timeZoneForDateAndTime


! !

!DateAndTime class methodsFor: 'smalltalk-80' stamp: 'dtl 10/23/2004 18:57'!
fromSeconds: seconds
	"Answer a DateAndTime since the Squeak epoch: 1 January 1901"

	^ (PointInTime fromSmalltalkSeconds: seconds) asDateAndTime
! !

!DateAndTime class methodsFor: '*time-TimeZones' stamp: 'dtl 11/2/2004 15:40'!
fromSeconds: seconds offsetSeconds: offset
	"Answer a DateAndTime since the Squeak epoch: 1 January 1901"

	| since offsetDuration |
	since _ Duration days: SqueakEpoch hours: 0 minutes: 0 seconds: seconds.
	offsetDuration _ Duration seconds: offset.
	^ self basicNew
		ticks: since ticks offset: offsetDuration;
		yourself.
! !


!LocalTimeTransform methodsFor: 'timezone conversion' stamp: 'dtl 11/23/2004 11:40'!
transformDate: aDate time: aTime toTimeZone: aTimeZone
	"For aDate and aTime here, answer the corresponding date and time in aTimeZone."

	"(LocalTimeTransform for: 'America/Detroit')
		transformDate: Date today
		time: Time now
		toTimeZone: (LocalTimeTransform for: 'America/Los_Angeles')"

	^ (PointInTime date: aDate time: aTime timeZone: self) asDateAndTimeArrayForTimeZone: aTimeZone
! !

!LocalTimeTransform methodsFor: 'timezone conversion' stamp: 'dtl 11/23/2004 19:13'!
transformFromTimeZone: aTimeZone date: aDate time: aTime
	"For aDate and aTime in aTimeZone, answer the corresponding date and time here."

	"(LocalTimeTransform for: 'America/Detroit')
		transformFromTimeZone: (LocalTimeTransform for: 'America/Los_Angeles')
		date: Date today
		time: Time now"

	^ (PointInTime date: aDate time: aTime timeZone: aTimeZone) asDateAndTimeArrayForTimeZone: self! !


!LocalTimeTransform class methodsFor: 'instance creation' stamp: 'dtl 11/12/2004 17:53'!
forSystem
	"For Squeak, use the DateAndTime>>localTimeZone, which should normally be
	configured as a proxy pointing back to the local time tranform in the time
	zone database. On other platforms, just use the current local transform in
	the database."

	"LocalTimeTransform forSystem"

	| dtClass |
	^ (dtClass _ Smalltalk at: #DateAndTime ifAbsent: []) isNil
		ifTrue: [self here]
		ifFalse: [dtClass perform: #localTimeZone]

		! !


!PointInTime methodsFor: 'accessing' stamp: 'dtl 11/17/2004 20:03'!
currentLeapSeconds
	"Current number of leap seconds since the Posix epoch. This is independent
	of geographical time zone, but will be reported as zero if the local time
	transform does not recognize leap seconds."

	"PointInTime now currentLeapSeconds"

	^ LocalTimeTransform forSystem leapSecondsAt: self
! !

!PointInTime methodsFor: 'accessing' stamp: 'dtl 11/17/2004 20:04'!
currentOffsetSeconds
	"Current seconds offset from UTC in the local time zone. For a given
	time zone, the value will change at daylight savings time transitions."

	"PointInTime now currentOffsetSeconds"

	^ LocalTimeTransform forSystem localOffsetSecondsAt: self
! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 10/23/2004 10:42'!
asAbsoluteSeconds
	"Answer the number of atomic clock seconds elapsed since 01-Jan-1970 UTC."

	^ self absoluteTime! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 11/12/2004 17:54'!
asDateAndTime
	"PointInTime now asDateAndTime"

	^ self asDateAndTimeForTimeZone: LocalTimeTransform forSystem
! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 11/23/2004 11:40'!
asDateAndTimeArrayForTimeZone: aTimeZone

	"PointInTime now asDateAndTimeArrayForTimeZone: LocalTimeTransform here"

	| dt |
	dt _ self asDateAndTimeForTimeZone: aTimeZone.
	^ Array
		with: dt asDate
		with: dt asTime! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 10/11/2004 19:40'!
asDateAndTimeForTimeZone: aTimeZone

	"PointInTime now asDateAndTimeForTimeZone: LocalTimeTransform here"

	| s offsetSeconds |
	s _ self asSmalltalkSecondsForTimeZone: aTimeZone.
	offsetSeconds _ aTimeZone localOffsetSecondsAt: self.
	s isNil
		ifTrue: [^ nil]
		ifFalse: [^ DateAndTime fromSeconds: s offsetSeconds: offsetSeconds]! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 9/21/2004 06:49'!
asFloat

	^ absoluteTime asFloat! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 11/12/2004 17:54'!
asLocalSmalltalkSeconds
	"Answer the number of seconds since January 1, 1901 in the local time zone. This
	number is suitable for creating instances of Time and Date."

	^ self asSmalltalkSecondsForTimeZone: LocalTimeTransform forSystem! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 10/27/2004 20:01'!
asNumber

	^ absoluteTime! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 11/12/2004 17:54'!
asPosixSeconds
	"Answer the number of Posix seconds since 01 January, 1970 UTC.
	The count of Posix seconds varies from absolute seconds as leap seconds
	are applied to the calendar."

	"Array with: PointInTime now asAbsoluteSeconds with: PointInTime now asPosixSeconds"

	^ self asPosixSecondsForTimeZone: LocalTimeTransform forSystem! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 10/23/2004 11:23'!
asPosixSecondsForTimeZone: aTimeZone
	"Answer the number of Posix seconds since 01 January, 1970 UTC.
	The count of Posix seconds varies from absolute seconds as leap seconds
	are applied to the calendar."

	^ self absoluteTime - (aTimeZone leapSecondsAt: self)
! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 10/26/2004 20:41'!
asSmalltalkSecondsForTimeZone: aTimeZone
	"Answer the number of seconds since January 1, 1901 in aTimeZone. This
	number is suitable for creating instances of Time and Date."

	| offset leap |
	self absoluteTime isNil ifTrue: [^ nil].
	aTimeZone isNil ifTrue: [self notify: 'nil TimeZone'. ^ nil].
	offset _ aTimeZone localOffsetSecondsAt: self.
	leap _ aTimeZone leapSecondsAt: self.
	^ self absoluteTime + self class posixOffset + offset - leap! !

!PointInTime methodsFor: 'comparing' stamp: 'dtl 10/26/2004 20:04'!
< aPointInTime

	^ (aPointInTime species == self species)
		and: [self absoluteTime < aPointInTime absoluteTime]! !

!PointInTime methodsFor: 'comparing' stamp: 'dtl 10/23/2004 13:16'!
= aPointInTime

	^ (aPointInTime species == self species)
		and: [aPointInTime absoluteTime = self absoluteTime]! !

!PointInTime methodsFor: 'comparing' stamp: 'dtl 10/23/2004 13:17'!
hash

	^ self absoluteTime hash! !

!PointInTime methodsFor: 'arithmetic' stamp: 'dtl 10/26/2004 20:48'!
- aPointInTime
	"Answer the difference in seconds between two points in time. The result
	may be converted to a Duration with #asDuration, but this method answers
	seconds in order to avoid loss of precision."

	^ absoluteTime - aPointInTime absoluteTime! !


!PointInTime class methodsFor: 'documentation' stamp: 'dtl 11/9/2004 22:09'!
squeakSecondsClockInfo

	"(StringHolder new acceptContents: self squeakSecondsClockInfo; yourself)
		openLabel: 'Squeak seconds clock' "

	^
'The Squeak seconds clock represents the number of seconds since the Smalltalk epoch. The Smalltalk epoch is defined as 1 January 1901. The current value of this clock is available by evaluating Time class>>totalSeconds, which calls the ioSeconds() function in the Squeak virtual machine to obtain the current local time from the underlying operating system.

This is a wall-clock time, in the sense that the Squeak seconds clock corresponds to the setting of a clock on the wall located (geographically) in the current time zone. When a daylight savings time (DST) transition occurs, the hands on the wall clock are moved either forward or backward. Similarly, the Squeak seconds clock will jump forward or backward whenever a DST transition occurs. The current Time in Squeak will also jump forward and backward when DST transitions occur. Of course actual time proceeds forward relentlessly, but the display of time on the wall clock will have changed in order to make timekeeping more meaningful in the local time zone. Likewise, the displayed value of DateAndTime in Squeak will have suddenly adjusted itself to match the wall clock display.

In addition to DST transitions, leap seconds are occasionally added to or removed from the world calendar. In principle, the seconds hand on the wall clock is moved forward or back by one second whenever a leap second is defined. Likewise, the Squeak seconds clock is (or should be) adjusted forward or back when leap seconds occur.

It is also possible to define a notion of time as a magnitude, independent of all of the adjustments that are made to wall clocks in various locations around the world. Class PointInTime represents this universal clock, and uses the convention of describing a point in time as the total elapsed seconds since 1 January 1, 1970 in the UTC time zone. These are seconds as would be reported by a very accurate Caesium clock located in Greenwich, England. These seconds increase continuously without regard for geographic time zones, daylight savings time changes, or leap second adjustments used for local displays of time on wall clocks.

It is always possible to derive the value of the Squeak seconds clock from a PointInTime, given information about the local time zone offset and leap seconds that are in effect as of that PointIntTime. The TimeZoneDatabase provides this information. For any PointInTime and LocalTimeTransform (time zone), the local time zone offset and leap second count are known, and the Squeak seconds clock can be calculated.

However, the reverse transformation (Squeak seconds clock to PointInTime) is not possible in the general case. To understand why, consider the case of a Squeak system at 150 minutes (2 1/2 hours) after midnight on Sunday, 31 October, 2004 in an Eastern Standard Time time zone. This is a half hour after the "fall back" time zone change at the end of summer. Assuming that we have kept track of our local time zone and leap seconds, this occurs at the PointInTime 1099204222 seconds after 1 January, 1970. If we check the Squeak seconds clock at this point in time, it reports 3276639000 seconds.

This is all well and good, but exactly one hour earlier the DST transition had not yet happened. The PointInTime was 1099200622 seconds, and the Squeak seconds clock was 3276639000, exactly the same value as the Squeak seconds one hour later. Thus the Squeak seconds clock had the same value at two completely different points in time. Therefore there are periods of time during which it is not possible to determine which of two points in time is the correct PointInTime for the Squeak seconds clock.

It is of course possible to add a primitive to Squeak that makes the current PointInTime available directly. This, in conjunction with the information in the TimeZoneDatabase, provides enough information to do transformations from PointInTime to Squeak seconds and back without ambiguity. Alternatively, a primitive that reports current time (either UTC or local) along with current time zone offset provides the required information.

As a related note, operating systems may not report time zone offset correctly for points in time that occur outside of the current DST setting. But if TimeZoneDatabase is available in the Squeak image and a UTC seconds primitive is added, then Date and Time transformations can be reliably calculated for any PointInTime within the domain of the TimeZoneDatabase transforms.
'! !

!PointInTime class methodsFor: 'documentation' stamp: 'dtl 11/9/2004 22:01'!
utcClockInfo

	"(StringHolder new acceptContents: self utcClockInfo; yourself)
		openLabel: 'UTC seconds clock' "

	^
'UTC is a measure of time corresponding to the angular rotation of the earth. As such, it is not a measure of time as measured by an atomic clock. UTC time is kept in synchronization with atomic time by the occasional insertion (or removal) of leap seconds in the UTC clock, although historically other methods of adjusting the clock (e.g. "elastic seconds") have been used, and the term "UTC" has had various definitions over the years. The distinction between rotational time (UTC) and atomic time can often be ignored. However, for purposes of physics and astronomy, they are different concepts and the small numerical differences between the two are significant.

Posix seconds are similar to UTC seconds, but are defined in such a way that leap seconds are ignored. The Posix seconds clock is thus effectively a "wall clock" representation of seconds, for which the Posix clock is adjusted forward or back when a leap second occurs. This is similar to the seconds clock in early Unix systems, which were designed prior to the invention of leap seconds.

Class PointInTime is intended to represent atomic time, such that time can be treated as a magnitude. Posix time varies from an atomic seconds clock as leap seconds are applied. For this reason, the conversion from PointInTime to and from Posix seconds should show a difference corresponding to the number of leap seconds that have been applied since 1 January, 1970.

If leap seconds are ignored, then Posix time and PointInTime seconds produce the same count of seconds. Many time zone tables ignore leap seconds, and many computer operating systems likewise ignore the issue. For example, the LocalTimeTransform for "right/America/Detroit" in a TimeZoneDatabase contains leap second rules, while the "posix/America/Detroit" transform always answers zero leap seconds. Both refer to the same geographical time zone, and for practical purposes the "posix/America/Detroit" transform is an appropriate choice to describe the time zone rules for Detroit, Michigan.

The LocalTimeTransforms in the TimeZoneDatabase contain transforms with or without leap second tables. The "posix/America/Detroit" transform ignores leap seconds, and is appropriate for a platform that is configured according to Posix specifications, while the "right/America/Detroit" transform accounts for leap seconds and provided a more realistic count of actual elapsed seconds since the Posix epoch.

It should not be assumed that the operatings system on which Squeak is hosted will handle leap seconds consistently. BSD Unix systems (including FreeBSD and Mac OSX) define the time_t system clock to represent atomic clock seconds and providing time2posix() and posix2time() functions for converting to and from Posix seconds. This allows correct counting of actual seconds since the Posix epoch. Other Unix and Linux systems may ignore leap seconds, and other operating systems may also handle leap seconds differently.

For purposes of converting Time and Date in Squeak to and from PointInTime seconds, we assume that Squeak will report its second clock as Posix seconds offset by the current time zone offset, regardless of the operating system and virtual machine on which it is running.
'! !

!PointInTime class methodsFor: 'initialize-release' stamp: 'dtl 11/12/2004 19:17'!
initialize
	"Set the selector to use to provide a floating point representation with
	adequate precision. Some systems make a distinction between Float and
	Double, and others simply use a double precision implementation of Float
	for all cases. If available, ScaledDecimal may also be used."

	"PointInTime initialize"

	Platform _ self platformSymbol.
"	(Platform == #Squeak) ifTrue: [self numericRepresentationSelector: #asFloat]."
	(Platform == #Squeak) ifTrue: [self numericRepresentationSelector: #asScaledDecimal].
	(Platform == #VisualWorks) ifTrue: [self numericRepresentationSelector: #asDouble].
	(Platform == #'Smalltalk/X') ifTrue: [PointInTime numericRepresentationSelector: #asFloat]! !

!PointInTime class methodsFor: 'initialize-release' stamp: 'dtl 11/21/2004 08:37'!
numericRepresentationSelector: aClassSelector
	"This method is normally called during class initialization. A double precision
	floating point representation may be used (either Double or Float, depending
	on the Smalltalk platform). ScaledDecimal also may be used where availabile.
	To change the representation to an Integer (for example), inspect the following:
		PointInTime numericRepresentationSelector: #asInteger; now
	And to change back to normal for this platform, inspect this:
		PointInTime initialize; now"

	NumericRepresentationSelector _ aClassSelector! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 11/12/2004 17:55'!
date: aDate time: aTime
	"Answer a PointInTime corresponding to the aDate and aTime in the
	local time zone."

	"PointInTime date: Date today time: Time now"

	^ self date: aDate time: aTime timeZone: LocalTimeTransform forSystem
! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 11/23/2004 11:41'!
date: aDate time: aTime timeZone: aTimeZone
	"Answer a PointInTime corresponding to the aDate and aTime in the
	context of aTimeZone."

	"PointInTime date: Date today time: Time now timeZone: LocalTimeTransform here"

	^ self
		fromSmalltalkSeconds: aDate asSeconds + aTime asSeconds
		inTimeZone: aTimeZone! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 10/21/2004 05:31'!
fromAbsoluteSeconds: seconds
	"Answer aPointInTime at a number of seconds after 1-Jan-1970 UTC"

	"PointInTime fromPosixSeconds: ((Date today asSeconds) + (Time now asSeconds) + 14400)"

	^ super new absoluteTime: seconds! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 11/12/2004 17:55'!
fromPosixSeconds: seconds
	"Answer aPointInTime at a number of seconds after 1-Jan-1970 UTC.
	The count of Posix seconds varies from absolute seconds as leap seconds
	are applied to the calendar."

	"PointInTime fromPosixSeconds: ((Date today asSeconds) + (Time now asSeconds) + 14400)"

	^ self fromPosixSeconds: seconds inTimeZone: LocalTimeTransform forSystem! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 11/20/2004 12:00'!
fromPosixSeconds: seconds inTimeZone: aTimeZone
	"Answer aPointInTime at a number of seconds after 1-Jan-1970 UTC.
	The count of Posix seconds varies from absolute seconds as leap seconds
	are applied to the calendar."

	^ super new absoluteTime: (self absoluteSecondsAtPosixSeconds: seconds inTimeZone: aTimeZone)! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 10/23/2004 12:10'!
fromSeconds: seconds
	"Answer aPointInTime at a number of seconds after 1-Jan-1901 in the local time zone."

	"PointInTime fromSeconds: Time totalSeconds"

	^ self fromSmalltalkSeconds: seconds
! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 11/12/2004 17:55'!
fromSmalltalkSeconds: seconds
	"Answer aPointInTime at a number of seconds after 1-Jan-1901."

	"PointInTime fromSmalltalkSeconds: Time totalSeconds"

	^ self fromSmalltalkSeconds: seconds inTimeZone: LocalTimeTransform forSystem! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 10/23/2004 09:54'!
fromSmalltalkSeconds: seconds inTimeZone: aTimeZone
	"Answer aPointInTime at a number of seconds after 1-Jan-1901 in a time zone.
	The time zone offset is calculated as of aPointInTime, which may not be the
	same offset as is currently in effect for aTimeZone. Both time zone offsets and
	leap seconds are used in the conversion.
	There is no direct mapping of local Smalltalk seconds to PointInTime, because
	time zone offset is a function of PointInTime rather than local Smalltalk time.
	For this reason an interative method is used to converge on the correct value."

	"PointInTime fromSmalltalkSeconds: Time totalSeconds inTimeZone: LocalTimeTransform here"

	^ self fromAbsoluteSeconds:
		(self absoluteSecondsAtSmalltalkSeconds: seconds inTimeZone: aTimeZone)! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 10/23/2004 13:33'!
fromUtcSmalltalkSeconds: seconds
	"Answer aPointInTime at a number of seconds after 1-Jan-1901 UTC, with no
	leap second compensation. This is equivalent to #fromSmalltalkSeconds: for a
	Smalltalk system located in Greenwich, England with its time zone set to a Posix
	time zone that ignores leap seconds."

	"PointInTime fromUtcSmalltalkSeconds: Time totalSeconds"

	^ super new absoluteTime: (seconds - self posixOffset)
! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 11/21/2004 10:33'!
now

	"PointInTime now"

	^ self fromAbsoluteSeconds: self absoluteSecondsNow
! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/20/2004 11:58'!
absoluteSecondsAtPosixSeconds: posixSeconds inTimeZone: aTimeZone
	"Answer the count of absolute seconds corresponding to posixSeconds. If the local
	time zone does not account for leap seconds, then the result is posixSeconds.
	There is no direct mapping of Posix seconds to absolute seconds, because when
	a leap second is added to the calendar, there will be a Posix second that corresponds
	to two absolute seconds. For this reason an interative method is used to converge on
	the correct value.

	During a period of time after a leap second is inserted into the calendar, there
	may be two distinct values of PointInTime with the same local Posix seconds
	value. This method will answer only one of possibly two valid results.
	
	See also absoluteTimeAtSmalltalkSeconds:inTimeZone:, which has a similar problem
	with ambiguous conversion."

	"PointInTime absoluteSecondsAtPosixSeconds: 1098500000 inTimeZone: LocalTimeTransform here"

	| leap newTime |
	aTimeZone ifNil: [self notify: 'nil TimeZone'. ^ nil].
	(1 to: 10)
		inject: posixSeconds
		into: [:time :i |
			leap _ aTimeZone leapSecondsAt: time.
			newTime _ posixSeconds + leap.
			(newTime = time) ifTrue: [^ time].
			newTime].
	self error: 'algorithm does not converge'.
	^ nil! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/21/2000 19:00'!
absoluteSecondsAtSmalltalkSeconds: seconds inTimeZone: aTimeZone

	"PointInTime absoluteSecondsAtSmalltalkSeconds: Time totalSeconds inTimeZone: LocalTimeTransform here"

	| possibleValues |
	possibleValues _ self
		allAbsoluteSecondsCorrespondingTo: seconds
		inTimeZone: aTimeZone.
	(possibleValues size == 1)
		ifTrue: [^ possibleValues at: 1].
	(possibleValues size == 2)
		ifTrue:
			[(TimeZoneDstTransitionWatcher clockWatcher fallBackOccurredDuringHourBefore: seconds)
				ifFalse: [^ possibleValues at: 1]
				ifTrue: [^ possibleValues at: 2]].
	self error: 'expected either one or two valid absolute time values'
! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/20/2004 12:02'!
absoluteTimeAtSmalltalkSeconds: smalltalkSeconds inTimeZone: aTimeZone
	"Find an absolute time for which the Smalltalk seconds clock is smalltalkSeconds.
	Answer an array with absolute seconds, UTC offset, and leap seconds for this
	time in aTimeZone.
	
	There is no direct mapping of local Smalltalk seconds to PointInTime, because
	time zone offset is a function of PointInTime rather than local Smalltalk time.
	For this reason an interative method is used to converge on a correct value.

	During a period of time after a daylight savings time transition, there may
	be two distinct values of PointInTime with the same local Smalltalk seconds
	value. This method will answer only one of possibly two valid results.
	
	See also absoluteSecondsAtPosixSeconds:inTimeZone:, which has a similar problem
	with ambiguous conversion."

	"PointInTime absoluteTimeAtSmalltalkSeconds: Time totalSeconds inTimeZone: LocalTimeTransform here"

	| leap offset newTime |
	aTimeZone ifNil: [self notify: 'nil TimeZone'. ^ nil].
	(1 to: 10)
		inject: smalltalkSeconds - self posixOffset
		into: [:time :i |
			leap _ aTimeZone leapSecondsAt: time.
			offset _ aTimeZone localOffsetSecondsAt: time.
			newTime _ smalltalkSeconds - self posixOffset - offset + leap.
			(newTime = time)
				ifTrue: [^ Array with: time with: offset with: leap].
			newTime].
	self error: 'algorithm does not converge'.
	^ nil! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/18/2004 19:20'!
allAbsoluteSecondsCorrespondingTo: aLocalSmalltalkSecondsCount inTimeZone: aTimeZone
	"Answer all values of absolute seconds that correspond to aLocalSmalltalkSecondsCount.
	Assume that the daylight savings time transition for the local time zone involves
	setting the clock forward or backward by exactly 3600 seconds (one hour). Assume
	that any leap second transitions involve setting the clock forward or back by
	one second."

	"self allAbsoluteSecondsCorrespondingTo: 3276637199
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"
	"self allAbsoluteSecondsCorrespondingTo: 3276637200
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"
	"self allAbsoluteSecondsCorrespondingTo: 3276640799
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"
	"self allAbsoluteSecondsCorrespondingTo: 3276640800
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"

	| s |
	s _ self firstAbsoluteSecondsAtSmalltalkSeconds: aLocalSmalltalkSecondsCount inTimeZone: aTimeZone.
	^ (Array
		with: s - 3600
		with: s - 1
		with: s
		with: s + 1
		with: s + 3600)
			select: [:absoluteSecs |
				((PointInTime fromAbsoluteSeconds: absoluteSecs)
					asSmalltalkSecondsForTimeZone: aTimeZone) = aLocalSmalltalkSecondsCount]

! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/18/2004 19:20'!
allAbsoluteTimesCorrespondingTo: aLocalSmalltalkSecondsCount inTimeZone: aTimeZone
	"Answer all values of PointInTime that correspond to aLocalSmalltalkSecondsCount.
	Assume that the daylight savings time transition for the local time zone involves
	setting the clock forward or backward by exactly 3600 seconds (one hour). Assume
	that any leap second transitions involve setting the clock forward or back by
	one second."

	"self allAbsoluteTimesCorrespondingTo: 3276637199
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"
	"self allAbsoluteTimesCorrespondingTo: 3276637200
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"
	"self allAbsoluteTimesCorrespondingTo: 3276640799
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"
	"self allAbsoluteTimesCorrespondingTo: 3276640800
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"

	| s |
	s _ self firstAbsoluteSecondsAtSmalltalkSeconds: aLocalSmalltalkSecondsCount inTimeZone: aTimeZone.
	^ (Array
		with: s - 3600
		with: s - 1
		with: s
		with: s + 1
		with: s + 3600)
			select: [:absoluteSecs |
				((PointInTime fromAbsoluteSeconds: absoluteSecs)
					asSmalltalkSecondsForTimeZone: aTimeZone) = aLocalSmalltalkSecondsCount]
			thenCollect: [:abs | PointInTime fromAbsoluteSeconds: abs]
! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 10/31/2004 02:00'!
dateAndTimeArrayFromSmalltalkSeconds: seconds
	"This is from the Time class>>dateAndTimeFromSeconds method in Squeak,
	provided here for compability on other Smalltalk systems."

	"PointInTime dateAndTimeArrayFromSmalltalkSeconds: Time totalSeconds"

	| dt |
	dt _ (self fromSmalltalkSeconds: seconds) asDateAndTime.
	^ Array
		with: dt asDate
		with: dt asTime! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/18/2004 19:20'!
firstAbsoluteSecondsAtSmalltalkSeconds: localSeconds inTimeZone: aTimeZone
	"Answer the first of the possibly two values of absolute seconds corresponding to localSeconds."

	"PointInTime firstAbsoluteSecondsAtSmalltalkSeconds: Time totalSeconds inTimeZone: LocalTimeTransform here"

	^ (self absoluteTimeAtSmalltalkSeconds: localSeconds inTimeZone: aTimeZone) first! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/20/2004 13:23'!
posixSecondsAtSmalltalkSeconds: seconds
	"Answer the count of Posix seconds corresponding to local seconds in the local
	time zone."

	"PointInTime posixSecondsAtSmalltalkSeconds: Time totalSeconds"

	^ self posixSecondsAtSmalltalkSeconds: seconds inTimeZone: DateAndTime localTimeZone
! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/20/2004 13:23'!
posixSecondsAtSmalltalkSeconds: seconds inTimeZone: aTimeZone
	"Answer the count of Posix seconds corresponding to local seconds in aTimeZone."

	"PointInTime posixSecondsAtSmalltalkSeconds: Time totalSeconds inTimeZone: LocalTimeTransform here"

	| abs |
	abs _ self absoluteTimeAtSmalltalkSeconds: seconds inTimeZone: aTimeZone.
	^ abs first + abs second
! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 10/23/2004 09:52'!
tzOffsetAtSmalltalkSeconds: seconds inTimeZone: aTimeZone

	"PointInTime tzOffsetAtSmalltalkSeconds: Time totalSeconds inTimeZone: LocalTimeTransform here"

	^ (self absoluteTimeAtSmalltalkSeconds: seconds inTimeZone: aTimeZone) second! !

!PointInTime class methodsFor: 'private' stamp: 'dtl 9/19/2004 23:50'!
platform
	"Answer what version of Smalltalk we are using."

	"PointInTime platform"

	^ Platform
! !

!PointInTime class methodsFor: 'private' stamp: 'dtl 9/19/2004 23:56'!
platformSymbol
	"Answer what version of Smalltalk we are using."

	| versionString versionClass |
	(versionClass _ Smalltalk at: #SystemVersion ifAbsent: []) isNil
		ifTrue: [versionString _ Smalltalk version]
		ifFalse: [versionString _ versionClass current version].
	('Squeak*' match: versionString) ifTrue: [^ #Squeak].
	('VisualWorks*' match: versionString) ifTrue: [^ #VisualWorks].
	('*Smalltalk.st*' match: versionString) ifTrue: [^ #'Smalltalk/X'].
	^ #unknown! !

!PointInTime class methodsFor: 'private' stamp: 'dtl 11/12/2004 17:56'!
testFromSmalltalkSeconds
	"Answer 'OK' if PointInTime>>fromSmalltalkSeconds gives the right result, plus
	or minus one second to allow for clock updates while this test is running."

	"PointInTime testFromSmalltalkSeconds"

	| t1 t2 diff |
	t1 _ Time totalSeconds.
	t2 _ (PointInTime fromSmalltalkSeconds: t1 inTimeZone: LocalTimeTransform forSystem) asLocalSmalltalkSeconds.
	diff _ t1 - t2.
	(diff abs < 1)
		ifTrue: [^ 'OK']
		ifFalse: [self notify: 'test failed'.
				^ 'Failed']! !

!PointInTime class methodsFor: 'private-system clock' stamp: 'dtl 11/21/2004 11:04'!
absoluteSecondsNow
	"Answer the current time in absolute seconds"

	"PointInTime absoluteSecondsNow"

	| platform |
	platform _ self platform.
	platform == #Squeak
		ifTrue: [^ self getAbsoluteSecondsFromPluginOrLocalSecondsClock].
	platform == #VisualWorks
		ifTrue: [^ self getAbsoluteSecondsFromLocalSecondsClock].
	platform == #'Smalltalk/X'
		ifTrue: [^ self getAbsoluteSecondsForStx]! !

!PointInTime class methodsFor: 'private-system clock' stamp: 'dtl 11/21/2004 11:03'!
getAbsoluteSecondsForStx
	"This method is platform dependent, intended for Smalltalk/X only. Assume
	that the operating system provides UTC Posix seconds (this assumption may
	not be correct)."

	| osClass posixSeconds |
	osClass _ Smalltalk at: #OperatingSystem.
	posixSeconds _ osClass getOSTime.
	^ self absoluteSecondsAtPosixSeconds: posixSeconds inTimeZone: LocalTimeTransform here
! !

!PointInTime class methodsFor: 'private-system clock' stamp: 'dtl 11/22/2000 09:51'!
getAbsoluteSecondsFromLocalSecondsClock

	"PointInTime getAbsoluteSecondsFromLocalSecondsClock"

	| t |
	t _ Time totalSeconds.
	"Force update to DST transition watcher, may be required if close to transition time."
	TimeZoneDstTransitionWatcher clockWatcher updateLastTick: t.
	^ self
		absoluteSecondsAtSmalltalkSeconds: t
		inTimeZone: DateAndTime localTimeZone
! !

!PointInTime class methodsFor: 'private-system clock' stamp: 'dtl 11/21/2004 10:46'!
getAbsoluteSecondsFromPluginOrLocalSecondsClock
	"Use a plugin if available, otherwise derive absolute seconds from
	the local seconds clock."

	"PointInTime getAbsoluteSecondsFromPluginOrLocalSecondsClock"

	| primTime | 
	(primTime _ self primPosixTimeMicrosecondResolution) isNil
		ifTrue:
			[^ self getAbsoluteSecondsFromLocalSecondsClock]
		ifFalse:
			[^ self
				absoluteSecondsAtPosixSeconds: primTime
				inTimeZone: DateAndTime localTimeZone]
! !

!PointInTime class methodsFor: 'private-system clock' stamp: 'dtl 11/17/2004 06:26'!
getStxSeconds
        "This method is platform dependent, intended for Smalltalk/X only. Note
        that 'AbsoluteTime now asSeconds' produces correct results for Unix based
        Smalltalk/X. However, the asSeconds method is documented as strictly
        private, hence this (hopefully) safer implementation. An additional
        adjustment may be required to accommodate leap seconds."

        | classAbsoluteTime now |
        classAbsoluteTime _ Smalltalk at: #AbsoluteTime.
        now _ classAbsoluteTime now.
        ^ now - (classAbsoluteTime secondsSince1970: 0) + now utcOffset
! !

!PointInTime class methodsFor: 'primitives' stamp: 'dtl 11/17/2004 19:50'!
primLocalSecondsClockWithOffset
	"Array of two integers, with number of seconds since since Jan 1, 1970 UTC,
	and the local offset for the time zone in effect for this system."

	| result |
	<primitive: 'primitiveLocalSecondsClockWithOffset' module: 'TimeFunctions'>
	result _ Array new: 2.
	result at: 1 put: Time primSecondsClock.
	result at: 2 put: 0.
	^ result! !

!PointInTime class methodsFor: 'primitives' stamp: 'dtl 11/17/2004 19:51'!
primPosixTime
	"Number of seconds, expressed as a float, since since Jan 1, 1970 UTC."

	<primitive: 'primitivePosixTime' module: 'TimeFunctions'>
	^ nil! !

!PointInTime class methodsFor: 'primitives' stamp: 'dtl 11/17/2004 19:51'!
primPosixTimeAndLocalOffset
	"Array of two integers, with number of seconds since since Jan 1, 1970 UTC,
	and the local offset for the time zone in effect for this system."

	<primitive: 'primitivePosixTimeAndLocalOffset' module: 'TimeFunctions'>
	^ nil! !

!PointInTime class methodsFor: 'primitives' stamp: 'dtl 11/17/2004 06:21'!
primPosixTimeMicrosecondResolution
	"Number of seconds, expressed as a float, since since Jan 1, 1970 UTC."

	<primitive: 'primitivePosixTimeMicrosecondResolution' module: 'TimeFunctions'>
	^ nil! !


!PointInTimeNow methodsFor: 'updating' stamp: 'dtl 11/21/2004 11:07'!
setAbsoluteTime

	"PointInTime new setAbsoluteTime asLocalDateAndTime"

	self absoluteTime: (self class absoluteSecondsNow)
! !


!TimePlugin methodsFor: 'primitives - time' stamp: 'dtl 9/21/2004 23:18'!
primitivePosixTime
	"Answer Posix time in seconds expressed as a Float."

	| dt |
	self export: true.
	self var: #dt declareC: 'double dt'.
	dt _ self cCode: '(double) time(NULL)'.
	interpreterProxy pop: 1; pushFloat: dt! !

!TimePlugin methodsFor: 'primitives - time' stamp: 'dtl 9/21/2004 23:18'!
primitivePosixTimeAndLocalOffset
	"Answer an array with the Posix time and the current local offset, with
	Posix time expressed as a Float, and local offset expressed as an Integer.
	The two values are obtained simultaneously."

	| t timeStruct result timezone floatTimeOop offsetOop |
	self export: true.
	self var: #t declareC: 'time_t t'.
	self var: #timeStruct declareC: 'struct tm *timeStruct'.
	self var: #timezone declareC: 'extern long int timezone'.

	t _ self cCode: 'time(NULL)'.
	timeStruct _ self cCode: 'localtime(&t)'.
	"External variable timezone is set as side effect of localtime(3)"

	floatTimeOop _ interpreterProxy floatObjectOf: t.
	offsetOop _ interpreterProxy integerObjectOf: timezone.

	interpreterProxy pushRemappableOop: offsetOop.
	interpreterProxy pushRemappableOop: floatTimeOop.
	result _ interpreterProxy
			instantiateClass: interpreterProxy classArray
			indexableSize: 2.	
	floatTimeOop _ interpreterProxy popRemappableOop.
	offsetOop _ interpreterProxy popRemappableOop.

	interpreterProxy stObject: result at: 1 put: floatTimeOop.
	interpreterProxy stObject: result at: 2 put: offsetOop.
	interpreterProxy pop: 1; push: result

! !

!TimePlugin methodsFor: 'primitives - time' stamp: 'dtl 9/21/2004 23:18'!
primitivePosixTimeMicrosecondResolution
	"Answer Posix time as a float with microsecond precision, limited of course
	by the capability of the hardware. Use a gettimeofday() call to get current
	time. This may not be supported on some systems."

	| tv dt |
	self export: true.
	self var: #tv declareC: 'struct timeval tv'.
	self var: #dt declareC: 'double dt'.
	(self cCode: 'gettimeofday(&tv, NULL)')
		ifTrue: [interpreterProxy pop: 1; push: interpreterProxy nilObject]
		ifFalse: [dt _ self cCode: 'tv.tv_usec / 1000000.0 + tv.tv_sec'.
				interpreterProxy pop: 1; pushFloat: dt]

! !

!TimePlugin methodsFor: 'primitives - timezone' stamp: 'dtl 9/21/2004 23:18'!
primitiveGetDaylightFlag
	"Answer the daylight savings time flag at the present time"

	| t daylight |
	self export: true.
	self var: #t declareC: 'time_t t'.
	t _ self cCode: 'time(NULL)'.
	self cCode: 'localtime(&t)'.
	"External variable daylight is set as side effect of localtime(3)"
	interpreterProxy pop: 1; pushBool: (daylight ~= 0)! !

!TimePlugin methodsFor: 'primitives - timezone' stamp: 'dtl 9/21/2004 23:19'!
primitiveGetDaylightFlagAt
	"Answer the daylight savings time flag for a Posix time expressed as a Float."

	| t daylight |
	self export: true.
	self var: #t declareC: 'time_t t'.
	t _ self popFloat.	"Coersce to time_t."
	self cCode: 'localtime(&t)'.
	"External variable daylight is set as side effect of localtime(3)"
	interpreterProxy pop: 1; pushBool: (daylight ~= 0)! !

!TimePlugin methodsFor: 'primitives - timezone' stamp: 'dtl 9/21/2004 23:19'!
primitiveGetTimeZone
	"Answer a four element array of two time zone names (primary and secondary),
	daylight savings time flag, and seconds offset from UTC."

	| t timeStruct tz strOop1 strPtr1 strOop2 strPtr2 daylight timezone |
	self export: true.
	self var: #t declareC: 'time_t t'.
	self var: #timeStruct declareC: 'struct tm *timeStruct'.
	self var: #strPtr1 declareC: 'char * strPtr1'.
	self var: #strPtr2 declareC: 'char * strPtr2'.

	"Get timezone information from the operating system."
	t _ self cCode: 'time(NULL)'.
	timeStruct _ self cCode: 'localtime(&t)'.
	"External variable tzname[] is set as side effect of localtime(3)"

	"Instantiate an array of two strings."
	tz _ interpreterProxy
			instantiateClass: interpreterProxy classArray
			indexableSize: 4.	
	interpreterProxy pushRemappableOop: tz.
	strOop1 _ interpreterProxy
			instantiateClass: interpreterProxy classString
			indexableSize: (self cCode: 'strlen(tzname[0])').
	interpreterProxy pushRemappableOop: strOop1.
	strOop2 _ interpreterProxy
			instantiateClass: interpreterProxy classString
			indexableSize: (self cCode: 'strlen(tzname[1])').
	strOop1 _ interpreterProxy popRemappableOop.
	tz _ interpreterProxy popRemappableOop.

	interpreterProxy stObject: tz at: 1 put: strOop1.
	interpreterProxy stObject: tz at: 2 put: strOop2.
	strPtr1 _ interpreterProxy arrayValueOf: strOop1.
	strPtr2 _ interpreterProxy arrayValueOf: strOop2.

	"Copy the time zone names into the strings."
	self cCode: 'strcpy(strPtr1, tzname[0])'.
	self cCode: 'strcpy(strPtr2, tzname[1])'.

	"Daylight savings time flag, a boolean."
	(daylight ~= 0)
		ifTrue: [interpreterProxy stObject: tz at: 3 put: interpreterProxy trueObject]
		ifFalse: [interpreterProxy stObject: tz at: 3 put: interpreterProxy falseObject].

	"Seconds offset from UTC."
	interpreterProxy stObject: tz at: 4 put: (interpreterProxy integerObjectOf: timezone).

	"Answer the results array."
	interpreterProxy pop: 1; push: tz













! !

!TimePlugin methodsFor: 'primitives - timezone' stamp: 'dtl 9/21/2004 23:19'!
primitiveGetTimeZoneAt
	"Answer a four element array of two time zone names (primary and secondary),
	daylight savings time flag, and seconds offset from UTC. The values are obtained
	relative to the given Posix time, which is passed as a Float."

	| t timeStruct tz strOop1 strPtr1 strOop2 strPtr2 daylight timezone |
	self export: true.
	self var: #t declareC: 'time_t t'.
	self var: #timeStruct declareC: 'struct tm *timeStruct'.
	self var: #strPtr1 declareC: 'char * strPtr1'.
	self var: #strPtr2 declareC: 'char * strPtr2'.

	"Get timezone information from the operating system."
	t _ self popFloat.	"Coersce to time_t."
	timeStruct _ self cCode: 'localtime(&t)'.
	"External variable tzname[] is set as side effect of localtime(3)"

	"Instantiate an array of two strings."
	tz _ interpreterProxy
			instantiateClass: interpreterProxy classArray
			indexableSize: 4.	
	interpreterProxy pushRemappableOop: tz.
	strOop1 _ interpreterProxy
			instantiateClass: interpreterProxy classString
			indexableSize: (self cCode: 'strlen(tzname[0])').
	interpreterProxy pushRemappableOop: strOop1.
	strOop2 _ interpreterProxy
			instantiateClass: interpreterProxy classString
			indexableSize: (self cCode: 'strlen(tzname[1])').
	strOop1 _ interpreterProxy popRemappableOop.
	tz _ interpreterProxy popRemappableOop.

	interpreterProxy stObject: tz at: 1 put: strOop1.
	interpreterProxy stObject: tz at: 2 put: strOop2.
	strPtr1 _ interpreterProxy arrayValueOf: strOop1.
	strPtr2 _ interpreterProxy arrayValueOf: strOop2.

	"Copy the time zone names into the strings."
	self cCode: 'strcpy(strPtr1, tzname[0])'.
	self cCode: 'strcpy(strPtr2, tzname[1])'.

	"Daylight savings time flag, a boolean."
	(daylight ~= 0)
		ifTrue: [interpreterProxy stObject: tz at: 3 put: interpreterProxy trueObject]
		ifFalse: [interpreterProxy stObject: tz at: 3 put: interpreterProxy falseObject].

	"Seconds offset from UTC."
	interpreterProxy stObject: tz at: 4 put: (interpreterProxy integerObjectOf: timezone).

	"Answer the results array."
	interpreterProxy pop: 1; push: tz













! !

!TimePlugin methodsFor: 'primitives - timezone' stamp: 'dtl 9/21/2004 23:19'!
primitiveGetTimeZoneName
	"Answer the name of this time zone."

	| t timeStruct tz tzPtr |
	self export: true.
	self var: #t declareC: 'time_t t'.
	self var: #timeStruct declareC: 'struct tm *timeStruct'.
	self var: #tzPtr declareC: 'char * tzPtr'.

	t _ self cCode: 'time(NULL)'.
	timeStruct _ self cCode: 'localtime(&t)'.
	"External variable tzname[] is set as side effect of localtime(3)"

	tz _ interpreterProxy
			instantiateClass: interpreterProxy classString
			indexableSize: (self cCode: 'strlen(tzname[0])').
	tzPtr _ interpreterProxy arrayValueOf: tz.

	self cCode: 'strcpy(tzPtr, tzname[0])'.
	interpreterProxy pop: 1; push: tz













! !

!TimePlugin methodsFor: 'primitives - timezone' stamp: 'dtl 9/21/2004 23:19'!
primitiveGetTimeZoneSecondaryName
	"Answer the name of this time zone."

	| t timeStruct tz tzPtr |
	self export: true.
	self var: #t declareC: 'time_t t'.
	self var: #timeStruct declareC: 'struct tm *timeStruct'.
	self var: #tzPtr declareC: 'char * tzPtr'.

	t _ self cCode: 'time(NULL)'.
	timeStruct _ self cCode: 'localtime(&t)'.
	"External variable tzname[] is set as side effect of localtime(3)"

	tz _ interpreterProxy
			instantiateClass: interpreterProxy classString
			indexableSize: (self cCode: 'strlen(tzname[1])').
	tzPtr _ interpreterProxy arrayValueOf: tz.

	self cCode: 'strcpy(tzPtr, tzname[1])'.
	interpreterProxy pop: 1; push: tz













! !

!TimePlugin methodsFor: 'primitives - timezone' stamp: 'dtl 9/21/2004 23:19'!
primitiveLocalOffsetSeconds

	| t timezone |
	self export: true.
	self var: #t declareC: 'time_t t'.
	self var: #timezone declareC: 'extern long int timezone'.
	t _ self cCode: 'time(NULL)'.
	self cCode: 'localtime(&t)'.
	"External variable timezone is set as side effect of localtime(3)"
	interpreterProxy pop: 1.
	interpreterProxy pushInteger: timezone


! !

!TimePlugin methodsFor: 'primitives - timezone' stamp: 'dtl 9/21/2004 23:20'!
primitiveLocalOffsetSecondsAt
	"Answer the local offset in seconds for a Posix time expressed as a Float.
	For a given time zone, the offset may change as a function of absolute
	time, for example if daylight savings time is in effect."

	| t timezone |
	self export: true.
	self var: #t declareC: 'time_t t'.
	self var: #timeStruct declareC: 'struct tm *timeStruct'.
	self var: #timezone declareC: 'extern long int timezone'.
	t _ self popFloat.	"Coersce to time_t."
	self cCode: 'localtime(&t)'.
	interpreterProxy pop: 1; pushInteger: timezone
! !

!TimePlugin methodsFor: 'primitives - timezone' stamp: 'dtl 9/21/2004 23:20'!
primitiveLocalSecondsClockWithOffset
	"Answer an array with the local time expressed in the Smalltalk frame of
	reference, and with the current local offset. The two values are obtained
	simultaneously. The result array is instantiated in this primitive."

	| t timeStruct timezone result timeOop offsetOop |
	self export: true.
	self var: #t declareC: 'time_t t'.
	self var: #timeStruct declareC: 'struct tm *timeStruct'.
	self var: #timezone declareC: 'extern long int timezone'.

	t _ self cCode: 'time(NULL)'.
	timeStruct _ self cCode: 'localtime(&t)'.
	"External variable timezone is set as side effect of localtime(3)"

	timeOop _ interpreterProxy positive32BitIntegerFor: (t + self posixOffset - timezone).
	interpreterProxy pushRemappableOop: timeOop.
	offsetOop _ interpreterProxy integerObjectOf: timezone.
	interpreterProxy pushRemappableOop: offsetOop.
	result _ interpreterProxy
			instantiateClass: interpreterProxy classArray
			indexableSize: 2.	
	offsetOop _ interpreterProxy popRemappableOop.
	timeOop _ interpreterProxy popRemappableOop.
	interpreterProxy stObject: result at: 1 put: timeOop.
	interpreterProxy stObject: result at: 2 put: offsetOop.
	interpreterProxy pop: 1; push: result

! !


!TimePlugin class methodsFor: 'translation' stamp: 'dtl 9/21/2004 22:51'!
shouldBeTranslated

	^ true! !


!TimeTransformTest methodsFor: 'documentation' stamp: 'dtl 11/9/2004 21:57'!
squeakSecondsClockInfo

	"(StringHolder new acceptContents: self new squeakSecondsClockInfo; yourself)
		openLabel: 'Squeak seconds clock' "

	^ PointInTime squeakSecondsClockInfo
! !

!TimeTransformTest methodsFor: 'documentation' stamp: 'dtl 11/9/2004 21:58'!
utcClockInfo

	"(StringHolder new acceptContents: self new utcClockInfo; yourself)
		openLabel: 'UTC seconds clock' "

	^ PointInTime utcClockInfo
! !

!TimeTransformTest methodsFor: 'running' stamp: 'dtl 11/14/2004 16:33'!
setUp

	"Save the system time zone setting"
	saveLocalTransform _ TimeZoneDatabase systemDatabase defaultTimeZone.
	saveLocalTimeZone _ DateAndTime localTimeZone.
	saveLastTransitionTime _ TimeZoneDstTransitionWatcher clockWatcher transitionTick.

	"Configure for time zone proxy to the database"
	TimeZoneProxy setDynamicTransform.

	"Fall 2004 DST transition for right/America/Detroit, in atomic clock seconds"
	dstTransitionAbsoluteSeconds _ 1099202422.

	"Fall 2004 DST transition for posix/America/Detroit, in Posix seconds, ignoring leap seconds"
	dstTransitionPosixSeconds _ 1099202400.

	"Number of leap seconds as of the transition time"
	leapSeconds _ 22.
! !

!TimeTransformTest methodsFor: 'running' stamp: 'dtl 11/14/2004 16:21'!
tearDown
	"Restore the default time zone"

	TimeZoneDatabase systemDatabase defaultLocation: saveLocalTransform timeZoneName.
	DateAndTime localTimeZone: saveLocalTimeZone.
	TimeZoneDstTransitionWatcher clockWatcher transitionTick: saveLastTransitionTime

! !

!TimeTransformTest methodsFor: 'testing-TZ offset at DST transition' stamp: 'dtl 11/7/2004 17:21'!
testDaylightSavingsTimeOffset
	"Test the offset during a time period when DST is effect"

	"(self new setTestSelector: #testDaylightSavingsTimeOffset) debug"

	| secs dt offset expectedOffset |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	secs _ 3273571904. "A time during summer 2004"
	dt _ DateAndTime fromSeconds: secs.
	self assert: ('2004-09-2*' match: dt printString).
	offset _ dt offset.
	expectedOffset _ Duration days: 0 hours: -4 minutes: 0 seconds: 00.
	self assert: offset = expectedOffset.
	! !

!TimeTransformTest methodsFor: 'testing-TZ offset at DST transition' stamp: 'dtl 11/7/2004 17:21'!
testDaylightSavingsTimeOffsetAtTransitionTime
	"Test the offset at the exact second of the DST transition time"

	"(self new setTestSelector: #testDaylightSavingsTimeOffsetAtTransitionTime) debug"

	| pt offsetSeconds |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	pt _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds.
	offsetSeconds _ LocalTimeTransform here localOffsetSecondsAt: pt.
	self assert: offsetSeconds = (-5 * 3600).
	self assert: pt asDateAndTime offset asSeconds = offsetSeconds
! !

!TimeTransformTest methodsFor: 'testing-TZ offset at DST transition' stamp: 'dtl 11/7/2004 17:22'!
testDaylightSavingsTimeOffsetImmediatelyAfter
	"Test the offset during a time period when DST is no longer in effect"

	"(self new setTestSelector: #testDaylightSavingsTimeOffsetImmediatelyAfter) debug"

	| pt offsetSeconds |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	pt _ PointInTime fromAbsoluteSeconds: (dstTransitionAbsoluteSeconds + 1).
	offsetSeconds _ LocalTimeTransform here localOffsetSecondsAt: pt.
	self assert: offsetSeconds = (-5 * 3600).
	self assert: pt asDateAndTime offset asSeconds = offsetSeconds
! !

!TimeTransformTest methodsFor: 'testing-TZ offset at DST transition' stamp: 'dtl 11/7/2004 17:20'!
testDaylightSavingsTimeOffsetImmediatelyBefore
	"Test the offset during a time period when DST is effect"

	"(self new setTestSelector: #testDaylightSavingsTimeOffsetImmediatelyBefore) debug"

	| pt offsetSeconds |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	pt _ PointInTime fromAbsoluteSeconds: (dstTransitionAbsoluteSeconds - 1).
	offsetSeconds _ LocalTimeTransform here localOffsetSecondsAt: pt.
	self assert: offsetSeconds = (-4 * 3600).
	self assert: pt asDateAndTime offset asSeconds = offsetSeconds
! !

!TimeTransformTest methodsFor: 'testing-TZ offset at DST transition' stamp: 'dtl 11/7/2004 17:21'!
testDaylightSavingsTimeOffsetLongAfter
	"Test the offset during a time period when DST is no longer in effect"

	"(self new setTestSelector: #testDaylightSavingsTimeOffsetLongAfter) debug"

	| pt offsetSeconds |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	pt _ PointInTime fromAbsoluteSeconds: (dstTransitionAbsoluteSeconds + (3600 * 24 * 7)).
	offsetSeconds _ LocalTimeTransform here localOffsetSecondsAt: pt.
	self assert: offsetSeconds = (-5 * 3600).
	self assert: pt asDateAndTime offset asSeconds = offsetSeconds
! !

!TimeTransformTest methodsFor: 'testing-TZ offset at DST transition' stamp: 'dtl 11/7/2004 17:22'!
testDaylightSavingsTimeOffsetLongBefore
	"Test the offset during a time period when DST is effect"

	"(self new setTestSelector: #testDaylightSavingsTimeOffsetLongBefore) debug"

	| pt offsetSeconds |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	pt _ PointInTime fromAbsoluteSeconds: (dstTransitionAbsoluteSeconds - (3600 * 24 * 7)).
	offsetSeconds _ LocalTimeTransform here localOffsetSecondsAt: pt.
	self assert: offsetSeconds = (-4 * 3600).
	self assert: pt asDateAndTime offset asSeconds = offsetSeconds
! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'dtl 11/6/2004 22:27'!
testAbsoluteTimeToDateAndTime
	"Check conversion from absolute time to DateAndTime local seconds as we cross 
	a DST transition."

	"(self new setTestSelector: #testAbsoluteTimeToDateAndTime) debug"

	| interval expectedStrings dateAndTimeStrings |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	interval _ dstTransitionAbsoluteSeconds - 2 to: dstTransitionAbsoluteSeconds + 1.
	expectedStrings _ #(
		'2004-10-31T01:59:58-04:00'
		'2004-10-31T01:59:59-04:00'
		'2004-10-31T01:00:00-05:00'
		'2004-10-31T01:00:01-05:00').
	dateAndTimeStrings _ interval
				collect: [:abs | (PointInTime fromAbsoluteSeconds: abs) asDateAndTime printString].
	self assert: dateAndTimeStrings = expectedStrings! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'dtl 10/31/2004 17:44'!
testAbsoluteTimeToDateAndTimeSeconds
	"Check conversion from absolute time to DateAndTime local seconds as we cross 
	a DST transition."

	"(self new setTestSelector: #testAbsoluteTimeToDateAndTimeSeconds) debug"

	| interval expectedLocalSmalltalkSeconds dateAndTimes dateAndTimeSeconds |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	interval _ dstTransitionAbsoluteSeconds - 2 to: dstTransitionAbsoluteSeconds + 1.
	expectedLocalSmalltalkSeconds _ #(3276640798 3276640799 3276637200 3276637201).
	dateAndTimes _ interval
				collect: [:abs | (PointInTime fromAbsoluteSeconds: abs) asDateAndTime].
	"This conversion is ambiguous for the hour following DST transition"
	dateAndTimeSeconds _ dateAndTimes collect: [:dt | dt asSeconds].
	"The third and fourth elements of the array are ambiguous, may be off by 1 hour"
	self assert: dateAndTimeSeconds = expectedLocalSmalltalkSeconds
! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'dtl 10/24/2004 21:47'!
testAbsoluteTimeToLocalSmalltalkSeconds
	"Check conversion from absolute time to local seconds count as we cross 
	a DST transition."

	"(self new setTestSelector: #testAbsoluteTimeToLocalSmalltalkSeconds) debug"

	| interval localSmalltalkSeconds expectedLocalSmalltalkSeconds |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	interval _ dstTransitionAbsoluteSeconds - 2 to: dstTransitionAbsoluteSeconds + 1.
	localSmalltalkSeconds _ interval
				collect: [:abs | (PointInTime fromAbsoluteSeconds: abs) asLocalSmalltalkSeconds].
	expectedLocalSmalltalkSeconds _ #(3276640798 3276640799 3276637200 3276637201).
	self assert: localSmalltalkSeconds = expectedLocalSmalltalkSeconds.
! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'dtl 11/14/2004 18:29'!
testConversionToLocalSmalltalkSeconds
	"Check conversion of PointInTime to local Smalltalk wall clock seconds.
	The Smalltalk seconds clock reports seconds in wall clock time in the local
	time zone, and is unaware of time zone offsets and leap seconds. The
	PointInTime clock reports actual elapsed Caesium clock seconds since
	the Posix epoch. As leap seconds occur, the two clocks become out of
	sync. If the local time transform is aware of leap seconds, the difference
	between the two clocks will appear in the conversion between PointInTime
	seconds and Smalltalk seconds."

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatsbase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testConversionToLocalSmalltalkSeconds) debug"

	| pt tSec tpSec absoluteSeconds leap posixSeconds offset calculatedSmalltalkSeconds |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	pt _ PointInTime now.
	tSec _ Time totalSeconds.
	tpSec _pt asLocalSmalltalkSeconds.
	self assert: (tSec - tpSec) abs < 1.

	absoluteSeconds _ pt asAbsoluteSeconds.
	leap _ LocalTimeTransform here leapSecondsAt: pt.
	posixSeconds _ absoluteSeconds + PointInTime posixOffset - leap.
	offset _ LocalTimeTransform here localOffsetSecondsAt: pt.
	calculatedSmalltalkSeconds _ posixSeconds + offset.
	self assert: (tSec - calculatedSmalltalkSeconds) abs < 1.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	pt _ PointInTime now.
	tSec _ Time totalSeconds.
	tpSec _pt asLocalSmalltalkSeconds.
	self assert: (tSec - tpSec) abs < 1.

	absoluteSeconds _ pt asAbsoluteSeconds.
	leap _ LocalTimeTransform here leapSecondsAt: pt.
	posixSeconds _ absoluteSeconds + PointInTime posixOffset - leap.
	offset _ LocalTimeTransform here localOffsetSecondsAt: pt.
	calculatedSmalltalkSeconds _ posixSeconds + offset.
	self assert: (tSec - calculatedSmalltalkSeconds) abs < 1

! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'dtl 11/6/2004 22:27'!
testConversionToLocalSmalltalkSeconds2
	"Check conversion of PointInTime to local Smalltalk wall clock seconds.
	The Smalltalk seconds clock reports seconds in wall clock time in the local
	time zone, and is unaware of time zone offsets and leap seconds. The
	PointInTime clock reports actual elapsed Caesium clock seconds since
	the Posix epoch. As leap seconds occur, the two clocks become out of
	sync. If the local time transform is aware of leap seconds, the difference
	between the two clocks will appear in the conversion between PointInTime
	seconds and Smalltalk seconds."

	| tpSec leap |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	tpSec _ 1098000000. "2004-10-17T04:00:00-04:00"
	leap _ LocalTimeTransform here leapSecondsAt: tpSec.
	self assert: leap == 22

! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'dtl 11/14/2004 16:25'!
testConversionToLocalSmalltalkSecondsNoLeapSeconds
	"Check conversion of PointInTime to local Smalltalk wall clock seconds.
	This test uses a time zone with no leap second table, so the conversion
	is expected to be exact."

	"(self new setTestSelector: #testConversionToLocalSmalltalkSecondsNoLeapSeconds) debug"

	| tpSec tSec |

	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	tpSec _ PointInTime now asLocalSmalltalkSeconds.
	tSec _ Time totalSeconds.
	"The values may be slightly different, but less than one second"
	self assert: (tSec - tpSec) < 1.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	tpSec _ PointInTime now asLocalSmalltalkSeconds.
	tSec _ Time totalSeconds.
	"The values may be slightly different, but less than one second"
	self assert: (tSec - tpSec) < 1.
! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'dtl 11/6/2004 22:28'!
testConvertLocalSecondsToPointInTime
	"Verify offset between UTC and America/Detroit, and leap second difference
	between America/Detroit and right/America/Detroit."

	| s pt1 pt2 pt3 |
	s _ 3275329734. "2004-10-15T17:48:54-04:00"

	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	pt1 _ PointInTime fromSeconds: s.
	self assert: s = pt1 asLocalSmalltalkSeconds.

	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	pt2 _ PointInTime fromSeconds: s.
	self assert: s = pt2 asLocalSmalltalkSeconds.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	pt3 _ PointInTime fromSeconds: s.
	self assert: s = pt3 asLocalSmalltalkSeconds.

	self assert: (pt2 absoluteTime - pt3 absoluteTime) = 14400. "time zone offset"
	self assert: (pt1 absoluteTime - pt2 absoluteTime) = 22. "leap seconds"! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'dtl 11/7/2004 20:25'!
testDateAndTimeSummer2004EST
	"Check 2004-09-05T12:00:00-04:00 using Posix seconds, absolute seconds, and local Smalltalk seconds."

	"(self new setTestSelector: #testDateAndTimeSummer2004EST) debug"

	| posixSeconds absoluteSeconds ptPosix ptAbsolute offset leap smalltalkSeconds ptSmalltalk |

	"Test with a time transform that includes leap seconds"
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	offset _ -4 * 3600.
	leap _ 22.

	"2004-09-05T16:00:00+00:00 Posix time, ignoring leap seconds"
	posixSeconds _ 1094400000.
	"Add leap seconds for the number of elapsed atomic clock seconds"
	absoluteSeconds _ posixSeconds + leap.
	"Adjust by difference in Posix epoch versus Smalltalk epoch, plus time zone offset"
	smalltalkSeconds _ posixSeconds + self posixOffset + offset.

	ptPosix _ PointInTime fromPosixSeconds: posixSeconds.
	ptAbsolute _ PointInTime fromAbsoluteSeconds: absoluteSeconds.
	ptSmalltalk _ PointInTime fromSmalltalkSeconds: smalltalkSeconds.

	"The absolute times should all be the same"
	self assert: ptPosix absoluteTime = ptAbsolute absoluteTime.
	self assert: ptSmalltalk absoluteTime = ptAbsolute absoluteTime.

	"Check commutative conversions"
	self assert: ptAbsolute = (PointInTime fromAbsoluteSeconds: ptAbsolute asAbsoluteSeconds).
	self assert: ptAbsolute = (PointInTime fromPosixSeconds: ptAbsolute asPosixSeconds).
	self assert: ptAbsolute = (PointInTime fromSmalltalkSeconds: ptAbsolute asLocalSmalltalkSeconds).

	"Run the same tests for a time transform that ignores leap seconds"
	TimeZoneDatabase systemDatabase defaultLocation: 'posix/America/Detroit'.
	offset _ -4 * 3600.
	leap _ 0.

	"2004-09-05T16:00:00+00:00 Posix time, ignoring leap seconds"
	posixSeconds _ 1094400000.
	"Add leap seconds for the number of elapsed atomic clock seconds"
	absoluteSeconds _ posixSeconds + leap.
	"Adjust by difference in Posix epoch versus Smalltalk epoch, plus time zone offset"
	smalltalkSeconds _ posixSeconds + self posixOffset + offset.

	ptPosix _ PointInTime fromPosixSeconds: posixSeconds.
	ptAbsolute _ PointInTime fromAbsoluteSeconds: absoluteSeconds.
	ptSmalltalk _ PointInTime fromSmalltalkSeconds: smalltalkSeconds.

	"The absolute times should all be the same"
	self assert: ptPosix absoluteTime = ptAbsolute absoluteTime.
	self assert: ptSmalltalk absoluteTime = ptAbsolute absoluteTime.

	"Check commutative conversions"
	self assert: ptAbsolute = (PointInTime fromAbsoluteSeconds: ptAbsolute asAbsoluteSeconds).
	self assert: ptAbsolute = (PointInTime fromPosixSeconds: ptAbsolute asPosixSeconds).
	self assert: ptAbsolute = (PointInTime fromSmalltalkSeconds: ptAbsolute asLocalSmalltalkSeconds).
! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'dtl 10/19/2004 23:06'!
testLeapSecondCount
	"Count of leap seconds as of 17 Oct, 2004 should be 22 for a time zone with leap
	second table. The UTC time zone does not have a leap second table, so the leap
	second count is zero."

	| tpSec leap |
	tpSec _ 1098000000.

	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	leap _ LocalTimeTransform here leapSecondsAt: tpSec.
	self assert: ((PointInTime fromPosixSeconds: tpSec) asDateAndTime year) = 2004.
	self assert: leap == 22.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	leap _ LocalTimeTransform here leapSecondsAt: tpSec.
	self assert: ((PointInTime fromPosixSeconds: tpSec) asDateAndTime year) = 2004.
	self assert: leap == 0.

! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'dtl 11/14/2004 18:30'!
testPointInTimeNowSeconds
	"Check consistency of various views on the current time"

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatsbase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testPointInTimeNowSeconds) debug"

	| clockSeconds pt1 pt2 ptSeconds1 ptSeconds2 pt3 pt4 ptSeconds3 ptSeconds4 |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	clockSeconds _ Time totalSeconds.
	pt1 _ PointInTime now.
	pt2 _ PointInTime fromSmalltalkSeconds: clockSeconds.
	ptSeconds1 _ pt1 asLocalSmalltalkSeconds.
	ptSeconds2 _ pt2 asLocalSmalltalkSeconds.
	self assert: (clockSeconds - ptSeconds1) abs < 1.
	self assert: (ptSeconds2 - ptSeconds1) abs < 1.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	clockSeconds _ Time totalSeconds.
	pt3 _ PointInTime now.
	pt4 _ PointInTime fromSmalltalkSeconds: clockSeconds.
	ptSeconds3 _ pt3 asLocalSmalltalkSeconds.
	ptSeconds4 _ pt4 asLocalSmalltalkSeconds.
	self assert: (clockSeconds - ptSeconds3) abs < 1.
	self assert: (ptSeconds4 - ptSeconds3) abs < 1.

	self assert: (ptSeconds4 - ptSeconds2) abs < 1.
	self assert: (ptSeconds3 - ptSeconds1) abs < 1.
! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'dtl 11/7/2004 22:44'!
testSmalltalkEpoch
	"Confirm that Smalltalk seconds are interpreted as wall clock time in the
	local time zone."

	"(self new setTestSelector: #testSmalltalkEpoch) debug"

	| smalltalkOriginDetroit smalltalkOriginUtc difference |

	TimeZoneDatabase systemDatabase defaultLocation: 'posix/America/Detroit'.
	smalltalkOriginDetroit _ DateAndTime fromSeconds: 0.

	TimeZoneDatabase systemDatabase defaultLocation: 'posix/UTC'.
	smalltalkOriginUtc _ DateAndTime fromSeconds: 0.

	self assert: smalltalkOriginDetroit > smalltalkOriginUtc.

	difference _ smalltalkOriginDetroit - smalltalkOriginUtc.
	self assert: difference asSeconds = (3600 * 6)
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 10/23/2004 14:43'!
testOffsetAsUtc
	"Check the right/America/Detroit transition time expressed as UTC (where UTC does not account for leap seconds)."

	"(self new setTestSelector: #testOffsetAsUtc) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	p _	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'UTC').
	"transition time should be '31 October 2004 6:00:22 am UTC'"
	self assert: p printString = '2004-10-31T06:00:22+00:00'.
	self assert: p offset = (Duration seconds: 0).

! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 10/23/2004 15:31'!
testOffsetAsUtcWithLeapSeconds
	"Check the right/America/Detroit and posix/America/Detroit transition time expressed as UTC.
	The transition time should be '31 October 2004 6:00:00 UTC.'"

	"(self new setTestSelector: #testOffsetAsUtcWithLeapSeconds) debug"

	| p1 p2 dt |
	"Test with a leap second aware local time zone"
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	p1 _ PointInTime fromPosixSeconds: dstTransitionPosixSeconds.
	dt _	p1 asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'right/UTC').
	self assert: dt printString = '2004-10-31T06:00:00+00:00'.
	self assert: dt offset = (Duration seconds: 0).
	dt _	p1 asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'posix/UTC').
	self assert: dt printString = '2004-10-31T06:00:22+00:00'.
	self assert: dt offset = (Duration seconds: 0).

	"Test with a local time zone that ignores leap seconds"
	TimeZoneDatabase systemDatabase defaultLocation: 'posix/America/Detroit'.
	p2 _ PointInTime fromPosixSeconds: dstTransitionPosixSeconds.
	self assert: (p1 absoluteTime - p2 absoluteTime) = 22.
	dt _	p2 asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'posix/UTC').
	self assert: dt printString = '2004-10-31T06:00:00+00:00'.
	self assert: dt offset = (Duration seconds: 0).
	dt _	p2 asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'right/UTC').
	self assert: dt printString = '2004-10-31T05:59:38+00:00'.
	self assert: dt offset = (Duration seconds: 0).
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 11/6/2004 22:39'!
testPointInTimeAtDstTransition
	"Check DateAndTime at the first second of the Fall 2004 DST period in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeAtDstTransition) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	p _	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit').
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-10-31T01:00:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 11/6/2004 22:41'!
testPointInTimeAtDstTransitionNoLeapSeconds
	"Check DateAndTime at the first second of the Fall 2004 DST period in America/Detroit."

	"(self new setTestSelector: #testPointInTimeAtDstTransitionNoLeapSeconds) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	p _	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit').
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-10-31T01:00:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 11/6/2004 22:41'!
testPointInTimeImmediatelyAfter
	"Check DateAndTime for the first second after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeImmediatelyAfter) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	p _	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds + 1)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit').
	self assert: p seconds == 1.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-10-31T01:00:01-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 11/6/2004 22:41'!
testPointInTimeImmediatelyAfterNoLeapSeconds
	"Check DateAndTime for the first second after the Fall 2004 DST transition in America/Detroit."

	"(self new setTestSelector: #testPointInTimeImmediatelyAfterNoLeapSeconds) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	p _	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds + 1)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit').
	self assert: p seconds == 1.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-10-31T01:00:01-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 11/6/2004 22:42'!
testPointInTimeImmediatelyBefore
	"Check DateAndTime for the second immediately prior to the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeImmediatelyBefore) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	p _	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds - 1)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit').
	self assert: p seconds == 59.
	self assert: p minutes == 59.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -4 * 60 * 60).
	self assert: p printString = '2004-10-31T01:59:59-04:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 11/6/2004 22:42'!
testPointInTimeImmediatelyBeforeNoLeapSeconds
	"Check DateAndTime for the second immediately prior to the Fall 2004 DST transition in America/Detroit."

	"(self new setTestSelector: #testPointInTimeImmediatelyBeforeNoLeapSeconds) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	p _	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds - 1)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit').
	self assert: p seconds == 59.
	self assert: p minutes == 59.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -4 * 60 * 60).
	self assert: p printString = '2004-10-31T01:59:59-04:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 11/6/2004 22:42'!
testPointInTimeOneDayAfter
	"Check DateAndTime for 24 hours after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeOneDayAfter) debug"

	| p oneDay |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	oneDay _ 3600 * 24.
	p _	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds + oneDay)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit').
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 11.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453311.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-11-01T01:00:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 11/6/2004 22:42'!
testPointInTimeOneDayBefore
	"Check DateAndTime for 24 hours prior to the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeOneDayBefore) debug"

	| p oneDay |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	oneDay _ 3600 * 24.
	p _	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds - oneDay)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit').
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 2.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453309.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -4 * 60 * 60).
	self assert: p printString = '2004-10-30T02:00:00-04:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 11/6/2004 22:42'!
testPointInTimeOneHalfHourAfter
	"Check DateAndTime for one half hour after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeOneHalfHourAfter) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	p _	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds + 1800)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit').
	self assert: p seconds == 0.
	self assert: p minutes == 30.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-10-31T01:30:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 11/6/2004 22:42'!
testPointInTimeOneHourAfter
	"Check DateAndTime for one hour after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeOneHourAfter) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	p _	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds + 3600)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit').
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 2.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-10-31T02:00:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'dtl 11/6/2004 22:42'!
testPointInTimeOneHourBefore
	"Check DateAndTime for one hour prior to the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeOneHourBefore) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	p _	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds - 3600)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit').
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -4 * 60 * 60).
	self assert: p printString = '2004-10-31T01:00:00-04:00'
! !

!TimeTransformTest methodsFor: 'testing-Posix seconds' stamp: 'dtl 10/23/2004 13:31'!
testPointInTimePosixOffset
	"Verify difference in seconds between Posix epoch and Smalltalk epoch GMT."

	| tp ts diff |
	tp _ PointInTime fromPosixSeconds: 0.
	ts _ PointInTime fromUtcSmalltalkSeconds: 0.
	self assert: PointInTime posixOffset = self posixOffset.
	diff _ tp asInteger - ts asInteger.
	self assert: self posixOffset = diff
! !

!TimeTransformTest methodsFor: 'testing-Posix seconds' stamp: 'dtl 11/20/2004 11:24'!
testPosixSecondsVersusAtomicSeconds
	"Test elapsed atomic clock seconds versus elapsed Posix seconds."

	"(self new setTestSelector: #testPosixSecondsVersusAtomicSeconds) debug"

	| absoluteSeconds posixSeconds pt1 pt2 diff |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	absoluteSeconds _ 1099202422. "Atomic clock seconds since January 1, 1970 UTC"
	posixSeconds _ 1099202400. "Posix clock seconds since January 1, 1970 UTC"
	pt1 _ PointInTime fromAbsoluteSeconds: absoluteSeconds.
	pt2 _ PointInTime fromPosixSeconds: posixSeconds.
	diff _ pt1 - pt2.
	self assert: diff abs < 1.
	self assert: pt1 asAbsoluteSeconds = absoluteSeconds.
	self assert: pt2 asPosixSeconds = posixSeconds.

	absoluteSeconds _ 0. "Atomic clock seconds at January 1, 1970 UTC"
	posixSeconds _ 0. "Posix clock seconds at January 1, 1970 UTC"
	pt1 _ PointInTime fromAbsoluteSeconds: absoluteSeconds.
	pt2 _ PointInTime fromPosixSeconds: posixSeconds.
	diff _ pt1 - pt2.
	self assert: diff abs < 1.
	self assert: pt1 asAbsoluteSeconds = absoluteSeconds.
	self assert: pt2 asPosixSeconds = posixSeconds.
! !

!TimeTransformTest methodsFor: 'testing-Posix seconds' stamp: 'dtl 10/30/2004 19:43'!
testPosixToPointInTimeConversion
	"Posix seconds should differ from PointInTime seconds by the number of leap
	seconds since the Posix epoch."

	"(self new setTestSelector: #testPosixToPointInTimeConversion) debug"

	| ptFromPosix ptFromAbsolute ptSec |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	ptFromPosix _ PointInTime fromPosixSeconds: dstTransitionPosixSeconds - (2 * 3600). "midnight"
	ptFromAbsolute _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (2 * 3600). "midnight"
	self assert: ptFromPosix = ptFromAbsolute.
	ptSec _ ptFromAbsolute absoluteTime.
	self assert: 2 * 3600 + ptSec = dstTransitionAbsoluteSeconds
! !

!TimeTransformTest methodsFor: 'testing-Posix seconds' stamp: 'dtl 10/23/2004 14:41'!
testPosixToPointInTimeConversionNoLeapSeconds
	"Posix seconds should differ from PointInTime seconds by the number of leap
	seconds since the Posix epoch, zero for a time zone with no leap second table."

	| pt ptSec |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	pt _ PointInTime fromPosixSeconds: dstTransitionPosixSeconds - (2 * 3600). "midnight"
	ptSec _ pt absoluteTime.
	self assert: (dstTransitionPosixSeconds - ptSec) = (2 * 3600)
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 21:49'!
testDateAndTimeAtDstTransition
	"Check DateAndTime at the first second of the Fall 2004 DST period in right/America/Detroit.
	The first Smalltalk second count for the transition occurs two hours after midnight, at
	which time the wall clock is set back to 1:00am."

	"(self new setTestSelector: #testDateAndTimeAtDstTransition) debug"

	| dt localSecondCount |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	localSecondCount _ self localSmalltalkDstTransition.
	self setDstTransitionWatcherFall2004.
	dt _ DateAndTime fromSeconds: localSecondCount.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:00:00-05:00'.
	self assert: (PointInTime fromSmalltalkSeconds: localSecondCount)
		asLocalSmalltalkSeconds = localSecondCount.
	self assert: dt asSeconds = localSecondCount.! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 21:50'!
testDateAndTimeAtDstTransitionNoLeapSeconds
	"Check DateAndTime at the first second of the Fall 2004 DST period in posix/America/Detroit.
	The first Smalltalk second count for the transition occurs two hours after midnight, at
	which time the wall clock is set back to 1:00am."

	"(self new setTestSelector: #testDateAndTimeAtDstTransitionNoLeapSeconds) debug"

	| dt localSecondCount |
	TimeZoneDatabase systemDatabase defaultLocation: 'posix/America/Detroit'.
	localSecondCount _ self localSmalltalkDstTransition.
	self setDstTransitionWatcherFall2004.
	dt _ DateAndTime fromSeconds: localSecondCount.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:00:00-05:00'.
	self assert: (PointInTime fromSmalltalkSeconds: localSecondCount)
		asLocalSmalltalkSeconds = localSecondCount.
	self assert: dt asSeconds = localSecondCount.! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 21:28'!
testDateAndTimeConversion
	"Check DateAndTime at midnight before the Fall 2004 DST period in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeConversion) debug"

	| midnight s dt1 dt2 |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	midnight _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (3600 * 2).
	s _ midnight asLocalSmalltalkSeconds.
	dt1 _ DateAndTime fromSeconds: s.
	dt2 _ midnight asDateAndTime.
	self assert: dt1 = dt2
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 21:51'!
testDateAndTimeImmediatelyAfter
	"Check DateAndTime for the first second after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeImmediatelyAfter) debug"

	| dt |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	self setDstTransitionWatcherFall2004.
	dt _ DateAndTime fromSeconds: self localSmalltalkDstTransition + 1.
	self assert: dt seconds == 1.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:00:01-05:00'
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 21:52'!
testDateAndTimeImmediatelyAfterNoLeapSeconds
	"Check DateAndTime for the first second after the Fall 2004 DST transition in America/Detroit."

	"(self new setTestSelector: #testDateAndTimeImmediatelyAfterNoLeapSeconds) debug"

	| dt |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	self setDstTransitionWatcherFall2004.
	dt _ DateAndTime fromSeconds: self localSmalltalkDstTransition + 1.
	self assert: dt seconds == 1.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:00:01-05:00'
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 21:33'!
testDateAndTimeImmediatelyBefore
	"Check DateAndTime for the second immediately prior to the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeImmediatelyBefore) debug"

	| dt |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	self setDstTransitionWatcherSpring2004.
	dt _ DateAndTime fromSeconds: self localSmalltalkDstTransition - 1.
	self assert: dt seconds == 59.
	self assert: dt minutes == 59.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -4 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:59:59-04:00'
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 21:34'!
testDateAndTimeImmediatelyBeforeNoLeapSeconds
	"Check DateAndTime for the second immediately prior to the Fall 2004 DST transition in posix/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeImmediatelyBeforeNoLeapSeconds) debug"

	| dt |
	TimeZoneDatabase systemDatabase defaultLocation: 'posix/America/Detroit'.
	self setDstTransitionWatcherSpring2004.
	dt _ DateAndTime fromSeconds: self localSmalltalkDstTransition - 1.
	self assert: dt seconds == 59.
	self assert: dt minutes == 59.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -4 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:59:59-04:00'
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 21:36'!
testDateAndTimeNowSeconds

	"(self new setTestSelector: #testDateAndTimeNowSeconds) debug"

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatsbase calculates
	Posix seconds from local seconds using the time zone offset."

	| clockSeconds dt dtSeconds |

	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	clockSeconds _ Time totalSeconds.
	dt _ DateAndTime now.
	dtSeconds _ dt asSeconds.
	self assert: (clockSeconds - dtSeconds) abs < 1.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	clockSeconds _ Time totalSeconds.
	dt _ DateAndTime now.
	dtSeconds _ dt asSeconds.
	self assert: (clockSeconds - dtSeconds) abs < 1! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 21:37'!
testDateAndTimeOffsetDuration

	"(self new setTestSelector: #testDateAndTimeOffsetDuration) debug"

	| local utc offsetSeconds tzOffsetSeconds tt |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	self setDstTransitionWatcherFall2004.
	tt _ self localSmalltalkDstTransition.
	local _ DateAndTime fromSeconds: tt.
	utc _ local asUTC.
	offsetSeconds _ local offset asSeconds - utc offset asSeconds.
	tzOffsetSeconds _ LocalTimeTransform here
		localOffsetSecondsAt: (PointInTime fromPosixSeconds: tt).
	self assert: offsetSeconds = tzOffsetSeconds
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/14/2004 16:43'!
testDateAndTimeOneDayAfter
	"Check DateAndTime for 24 hours after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeOneDayAfter) debug"

	| oneDay dt pt s |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	oneDay _ 3600 * 24.
	pt _ PointInTime fromAbsoluteSeconds: (dstTransitionAbsoluteSeconds + oneDay).
	s _ pt asLocalSmalltalkSeconds.
	dt _ DateAndTime fromSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 11.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453311.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-11-01T01:00:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/6/2004 22:43'!
testDateAndTimeOneDayBefore
	"Check DateAndTime for 24 hours prior to the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeOneDayBefore) debug"

	| oneDay dt pt s |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	oneDay _ 3600 * 24.
	pt _ PointInTime fromAbsoluteSeconds: (dstTransitionAbsoluteSeconds - oneDay).
	s _ pt asLocalSmalltalkSeconds.
	dt _ DateAndTime fromSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 2.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453309.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -4 * 60 * 60).
	self assert: dt printString = '2004-10-30T02:00:00-04:00'
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 21:55'!
testDateAndTimeOneHalfHourAfter
	"Check DateAndTime for one half hour after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeOneHalfHourAfter) debug"

	| dt pt s dt1 |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	self setDstTransitionWatcherFall2004.
	pt _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + 1800.
	dt _ pt asDateAndTime.
	s _ pt asLocalSmalltalkSeconds.
	dt1 _ DateAndTime fromSeconds: s.
	self assert: dt = dt1.
	self assert: dt1 asSeconds = dt asSeconds. "Same seconds count transformed back to local"
	"dt is the valid DateAndTime. Check that it has the expected value."
	self assert: dt seconds == 0.
	self assert: dt minutes == 30.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:30:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 21:39'!
testDateAndTimeOneHourAfter
	"Check DateAndTime for one hour after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeOneHourAfter) debug"

	| dt pt s |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	self setDstTransitionWatcherFall2004.
	pt _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + 3600.
	s _ pt asLocalSmalltalkSeconds.
	dt _ DateAndTime fromSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 2.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T02:00:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 21:40'!
testDateAndTimeOneHourBefore
	"Check DateAndTime for one hour prior to the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeOneHourBefore) debug"

	| dt pt s |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	self setDstTransitionWatcherSpring2004.
	pt _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (3600 * 1).
	s _ pt asLocalSmalltalkSeconds.
	dt _ DateAndTime fromSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -4 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:00:00-04:00'
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/6/2004 22:44'!
testDateAndTimeThreeHoursAfter
	"Check DateAndTime at three hours after the Fall 2004 DST period in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeThreeHoursAfter) debug"

	| dt pt s |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	pt _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (3600 * 3).
	s _ pt asLocalSmalltalkSeconds.
	dt _ DateAndTime fromSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 4.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T04:00:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/6/2004 22:44'!
testDateAndTimeTwoHoursAfter
	"Check DateAndTime at two hours after the Fall 2004 DST period in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeTwoHoursAfter) debug"

	| dt pt s |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	pt _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (3600 * 2).
	s _ pt asLocalSmalltalkSeconds.
	dt _ DateAndTime fromSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 3.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T03:00:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/6/2004 22:44'!
testDateAndTimeTwoHoursBefore
	"Check DateAndTime at midnight before the Fall 2004 DST period in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeTwoHoursBefore) debug"

	| dt midnight s |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	midnight _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (3600 * 2).
	s _ midnight asLocalSmalltalkSeconds.
	dt _ DateAndTime fromSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 0.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -4 * 60 * 60).
	self assert: dt printString = '2004-10-31T00:00:00-04:00'
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'dtl 11/23/2004 18:28'!
testDateFromYearMonthDay
	"Check DateAndTime creation in different DST periods."

	"(self new setTestSelector: #testDateFromYearMonthDay) debug"

	| dt1 dt2 |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	dt1 _ DateAndTime year: 2004 month: 'Nov' day: 9.
	self assert: dt1 year = 2004.
	self assert: dt1 month = 11.
	self assert: dt1 dayOfMonth = 9.
	self assert: dt1 hour = 0.
	self assert: dt1 minute = 0.
	self assert: dt1 second = 0.
	self assert: dt1 offset asSeconds = (3600 * -5).

	dt2 _ DateAndTime year: 2004 month: 'Aug' day: 9.
	self assert: dt2 year = 2004.
	self assert: dt2 month = 8.
	self assert: dt2 dayOfMonth = 9.
	self assert: dt2 hour = 0.
	self assert: dt2 minute = 0.
	self assert: dt2 second = 0.
	self assert: dt2 offset asSeconds = (3600 * -4).
! !

!TimeTransformTest methodsFor: 'testing-Date and Time array' stamp: 'dtl 11/23/2004 19:07'!
testCreateDateAndTimeArray

	"(self new setTestSelector: #testCreateDateAndTimeArray) debug"

	| date time z1 z2 z3 pt1 pt2 pt3 dt1 dt2 dt3 |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	date _ '23 Nov, 2004' asDate.
	time _ '12:00' asTime.
	z1 _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit'.
	z2 _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Chicago'.
	z3 _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Los_Angeles'.
	pt1 _ PointInTime date: date time: time timeZone: z1.
	pt2 _ PointInTime date: date time: time timeZone: z2.
	pt3 _ PointInTime date: date time: time timeZone: z3.
	dt1 _ pt1 asDateAndTimeForTimeZone: z1.
	dt2 _ pt2 asDateAndTimeForTimeZone: z2.
	dt3 _ pt3 asDateAndTimeForTimeZone: z3.
	self assert: dt1 = '2004-11-23T12:00:00-05:00' asDateAndTime.
	self assert: dt2 = '2004-11-23T12:00:00-06:00' asDateAndTime.
	self assert: dt3 = '2004-11-23T12:00:00-08:00' asDateAndTime.

	date _ '23 Aug, 2004' asDate. "Daylight savings time is in effect"
	time _ '12:00' asTime.
	z1 _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit'.
	z2 _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Chicago'.
	z3 _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Los_Angeles'.
	pt1 _ PointInTime date: date time: time timeZone: z1.
	pt2 _ PointInTime date: date time: time timeZone: z2.
	pt3 _ PointInTime date: date time: time timeZone: z3.
	dt1 _ pt1 asDateAndTimeForTimeZone: z1.
	dt2 _ pt2 asDateAndTimeForTimeZone: z2.
	dt3 _ pt3 asDateAndTimeForTimeZone: z3.
	self assert: dt1 = '2004-08-23T12:00:00-04:00' asDateAndTime.
	self assert: dt2 = '2004-08-23T12:00:00-05:00' asDateAndTime.
	self assert: dt3 = '2004-08-23T12:00:00-07:00' asDateAndTime.
! !

!TimeTransformTest methodsFor: 'testing-Date and Time array' stamp: 'dtl 11/23/2004 19:16'!
testTransformADateAndATimeFromTimeZone

	"(self new setTestSelector: #testTransformADateAndATimeFromTimeZone) debug"

	| date time z1 z2 z3 a |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	date _ '23 Nov, 2004' asDate.
	time _ '12:00' asTime.
	z1 _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit'.
	z2 _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Chicago'.
	z3 _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Los_Angeles'.
	a _ z1 transformFromTimeZone: z2 date: date time: time.
	self assert: a first = date.
	self assert: (a second asSeconds - time asSeconds) = 3600.
	a _ z1 transformFromTimeZone: z3 date: date time: time.
	self assert: a first = date.
	self assert: (a second asSeconds - time asSeconds) = (3 * 3600).
	a _ z2 transformFromTimeZone: z3 date: date time: time.
	self assert: a first = date.
	self assert: (a second asSeconds - time asSeconds) = (2 * 3600).
! !

!TimeTransformTest methodsFor: 'testing-Date and Time array' stamp: 'dtl 11/23/2004 12:12'!
testTransformADateAndATimeToTimeZone

	"(self new setTestSelector: #testTransformADateAndATimeToTimeZone) debug"

	| date time z1 z2 z3 a |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	date _ '23 Nov, 2004' asDate.
	time _ '12:00' asTime.
	z1 _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit'.
	z2 _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Chicago'.
	z3 _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Los_Angeles'.
	a _ z1 transformDate: date time: time toTimeZone: z2.
	self assert: a first = date.
	self assert: (time asSeconds - a second asSeconds) = 3600.
	a _ z1 transformDate: date time: time toTimeZone: z3.
	self assert: a first = date.
	self assert: (time asSeconds - a second asSeconds) = (3 * 3600).
	a _ z2 transformDate: date time: time toTimeZone: z3.
	self assert: a first = date.
	self assert: (time asSeconds - a second asSeconds) = (2 * 3600).
! !

!TimeTransformTest methodsFor: 'testing-local conversion' stamp: 'dtl 11/23/2004 21:14'!
testAmbiguousConversionAfterDst
	"Demonstrate ambiguity of translation from PointInTime to Smalltalk seconds during
	the DST transition."

	"(self new setTestSelector: #testAmbiguousConversionAfterDst) debug"

	| halfHourBefore halfHourAfter smalltalkSecondsBefore smalltalkSecondsAfter |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	halfHourBefore _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - 1800.
	halfHourAfter _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + 1800.
	self setDstTransitionWatcherSpring2004.
	smalltalkSecondsBefore _ halfHourBefore asLocalSmalltalkSeconds.
	self setDstTransitionWatcherFall2004.
	smalltalkSecondsAfter _ halfHourAfter asLocalSmalltalkSeconds.
	self deny: halfHourBefore = halfHourAfter.
	self assert: smalltalkSecondsBefore = smalltalkSecondsAfter
! !

!TimeTransformTest methodsFor: 'testing-local conversion' stamp: 'dtl 11/23/2004 21:17'!
testAmbiguousConversionForLeapSeconds
	"Demonstrate ambiguity of translation from PointInTime to Smalltalk seconds during
	a leap second transition."

	"(self new setTestSelector: #testAmbiguousConversionForLeapSeconds) debug"

	| tz leapSecondTransition oneSecondBeforeTransition ptArray twoSecondsAfterTransition |
	tz _ TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit'.
	leapSecondTransition _ (PointInTime fromAbsoluteSeconds: 915148821) asSmalltalkSecondsForTimeZone: tz.
	
	"One second before a leap second transition, conversion to local seconds is unambiguous."
	oneSecondBeforeTransition _ leapSecondTransition - 1.
	ptArray _ PointInTime allAbsoluteTimesCorrespondingTo: oneSecondBeforeTransition inTimeZone: tz.
	self assert: ptArray size == 1.
	self assert: (ptArray first asSmalltalkSecondsForTimeZone: tz) = oneSecondBeforeTransition.

	"During the two second duration of the leap second transition, conversion to local seconds is ambiguous.
	There are two atomic clock PointInTime seconds corresponding to a single wall clock second."
	ptArray _ PointInTime allAbsoluteTimesCorrespondingTo: leapSecondTransition inTimeZone: tz.
	self assert: ptArray size == 2.
	self deny: ptArray first = ptArray second.
	"But both of the PointInTime values translate back to the same local Smalltalk second count."
	self assert: (ptArray first asSmalltalkSecondsForTimeZone: tz) = leapSecondTransition.
	self assert: (ptArray second asSmalltalkSecondsForTimeZone: tz) = leapSecondTransition.

	"Two seconds after the leap seconds transition, conversion to local seconds is unambiguous again."
	twoSecondsAfterTransition _ leapSecondTransition + 1.
	ptArray _ PointInTime allAbsoluteTimesCorrespondingTo: twoSecondsAfterTransition inTimeZone: tz.
	self assert: ptArray size == 1.
	self assert: (ptArray first asSmalltalkSecondsForTimeZone: tz) = twoSecondsAfterTransition.

! !

!TimeTransformTest methodsFor: 'testing-local conversion' stamp: 'dtl 10/30/2004 18:41'!
testAmbiguousConversionToLocalSeconds
	"Demonstrate ambiguity of translation from PointInTime to Smalltalk seconds during
	the DST transition."

	"(self new setTestSelector: #testAmbiguousConversionToLocalSeconds) debug"

	"One second before the fall 2004 DST transition, conversion to local seconds is unambiguous."
	self assert:
		(PointInTime allAbsoluteTimesCorrespondingTo: 3276637199
			inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')) size == 1.

	"During the hour after the fall 2004 DST transition, conversion to local seconds is ambiguous."
	"There are two PointInTime values for a Smalltalk second count."
	self assert:
		(PointInTime allAbsoluteTimesCorrespondingTo: 3276637200
			inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')) size == 2.
	"But both of the PointInTime values translate back to the same local Smalltalk second count."
	self assert:
		((PointInTime allAbsoluteTimesCorrespondingTo: 3276637200
			inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit'))
				collect: [:e | e asSmalltalkSecondsForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')])
			asSet size == 1.
	"This continues to be the case for the next hour."
	self assert:
		(PointInTime allAbsoluteTimesCorrespondingTo: 3276640799
			inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')) size == 2.
	self assert:
		((PointInTime allAbsoluteTimesCorrespondingTo: 3276640799
			inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit'))
				collect: [:e | e asSmalltalkSecondsForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')])
			asSet size == 1.

	"One hour after the fall 2004 DST transition, conversion to local seconds is unambiguous again."
	self assert:
		(PointInTime allAbsoluteTimesCorrespondingTo: 3276640800
			inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')) size == 1.

! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'dtl 10/23/2004 22:48'!
testDateAtDstTransition
	"Check Date at a DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAtDstTransition) debug"

	| dt |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	dt _ Date fromSeconds: self localSmalltalkDstTransition.
	self assert: dt yyyymmdd = '2004-10-31'! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'dtl 10/23/2004 22:51'!
testDateAtDstTransitionNoLeapSeconds
	"Check Date at a DST transition in posix/America/Detroit."

	"(self new setTestSelector: #testDateAtDstTransitionNoLeapSeconds) debug"

	| dt |
	TimeZoneDatabase systemDatabase defaultLocation: 'posix/America/Detroit'.
	dt _ Date fromSeconds: self localSmalltalkDstTransition.
	self assert: dt yyyymmdd = '2004-10-31'! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'dtl 11/23/2004 22:06'!
testDateFirstSecondOfDayOfDstTransition
	"Check Date at the first second of the day on which a DST transition occurs
	in right/America/Detroit."

	"(self new setTestSelector: #testDateFirstSecondOfDayOfDstTransition) debug"

	| dt s |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	self setDstTransitionWatcherSpring2004.
	s _ self localSmalltalkDstTransition - (2 * 3600).
	dt _ Date fromSeconds: s.
	self assert: dt yyyymmdd = '2004-10-31'! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'dtl 11/23/2004 22:07'!
testDateFirstSecondOfDayOfDstTransitionNoLeapSeconds
	"Check Date at the first second of the day on which a DST transition occurs
	in posix/America/Detroit."

	"(self new setTestSelector: #testDateFirstSecondOfDayOfDstTransitionNoLeapSeconds) debug"

	| dt s |
	TimeZoneDatabase systemDatabase defaultLocation: 'posix/America/Detroit'.
	self setDstTransitionWatcherSpring2004.
	s _ self localSmalltalkDstTransition - (2 * 3600).
	dt _ Date fromSeconds: s.
	self assert: dt yyyymmdd = '2004-10-31'! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'dtl 11/23/2004 19:08'!
testDateInDstPeriods
	"Check the format of a Date posix/America/Detroit, before and after DST transition.
	The start times should show different UTC offsets before and after a DST
	transition. This is required for correct calculation of Date>>asSeconds."

	"(self new setTestSelector: #testDateInDstPeriods) debug"

	| d secondCount1 secondCount2 pt1 pt2 hourOffsetSmalltalkSeconds hourOffsetAbsoluteSeconds |
	TimeZoneDatabase systemDatabase defaultLocation: 'posix/America/Detroit'.
	d _ Date fromString: '23 Nov, 2004'.
	self assert: d start asString = '2004-11-23T00:00:00-05:00'.
	self assert: d duration asSeconds = (3600 * 24).
	pt1 _ d start asPointInTime.
	secondCount1 _ d asSeconds.

	d _ Date fromString: '23 Aug, 2004'.
	self assert: d start asString = '2004-08-23T00:00:00-04:00'.
	self assert: d duration asSeconds = (3600 * 24).
	pt2 _ d start asPointInTime.
	secondCount2 _ d asSeconds.
	hourOffsetSmalltalkSeconds _ (secondCount2 - secondCount1) \\ (24 * 3600).
	self assert: hourOffsetSmalltalkSeconds = 0.
	hourOffsetAbsoluteSeconds _ (pt1 asAbsoluteSeconds - pt2 asAbsoluteSeconds) \\ (24 * 3600).
	self assert: hourOffsetAbsoluteSeconds = 3600.

	d _ Date fromString: '23 Nov, 2003'.
	self assert: d start asString = '2003-11-23T00:00:00-05:00'.
	self assert: d duration asSeconds = (3600 * 24).

	d _ Date fromString: '23 Aug, 2003'.
	self assert: d start asString = '2003-08-23T00:00:00-04:00'.
	self assert: d duration asSeconds = (3600 * 24).

! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'dtl 10/23/2004 22:54'!
testDateLastSecondOfDayBeforeDstTransition
	"Check Date at the last second of the day before DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateLastSecondOfDayBeforeDstTransition) debug"

	| dt s |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	s _ self localSmalltalkDstTransition - (2 * 3600) - 1.
	dt _ Date fromSeconds: s.
	self assert: dt yyyymmdd = '2004-10-30'! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'dtl 10/23/2004 22:54'!
testDateLastSecondOfDayBeforeDstTransitionNoLeapSeconds
	"Check Date at the last second of the day before DST transition in posix/America/Detroit."

	"(self new setTestSelector: #testDateLastSecondOfDayBeforeDstTransitionNoLeapSeconds) debug"

	| dt s |
	TimeZoneDatabase systemDatabase defaultLocation: 'posix/America/Detroit'.
	s _ self localSmalltalkDstTransition - (2 * 3600) - 1.
	dt _ Date fromSeconds: s.
	self assert: dt yyyymmdd = '2004-10-30'! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'dtl 11/14/2004 16:30'!
testAsTime
	"Ensure that a Time created from a DateAndTime is the right magnitude."

	"(self new setTestSelector: #testAsTime) debug"

	| dt t dtSecs tSecs |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	dt _ DateAndTime now.
	t _ dt asTime.
	dtSecs _ (dt hour * 3600) + (dt minute * 60) + dt second.
	tSecs _ (t hour * 3600) + (t minute * 60) + t second.
	"dtSecs is a whole integer, so values are expected to match to within one second"
	self assert: (dtSecs - tSecs) abs < 1.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	dt _ DateAndTime now.
	t _ dt asTime.
	dtSecs _ (dt hour * 3600) + (dt minute * 60) + dt second.
	tSecs _ (t hour * 3600) + (t minute * 60) + t second.
	"dtSecs is a whole integer, so values are expected to match to within one second"
	self assert: (dtSecs - tSecs) abs < 1
! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'dtl 11/14/2004 16:30'!
testDateAndTimeFromSeconds
	"Test conversion using a time zone with a leap second table."

	| s dt dta |

	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	s _ Time totalSeconds.
	dt _ DateAndTime fromSeconds: s.
	dta _ Time dateAndTimeFromSeconds: s.
	self assert: dt asDate = dta first.
	self assert: dt asTime = dta last.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	s _ Time totalSeconds.
	dt _ DateAndTime fromSeconds: s.
	dta _ Time dateAndTimeFromSeconds: s.
	self assert: dt asDate = dta first.
	self assert: dt asTime = dta last
! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'dtl 11/14/2004 16:30'!
testDateAndTimeFromSecondsNoLeapSeconds
	"Test conversion using a time zone with no leap second table."

	| s dt dta |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	s _ Time totalSeconds.
	dt _ DateAndTime fromSeconds: s.
	dta _ Time dateAndTimeFromSeconds: s.
	self assert: dt asDate = dta first.
	self assert: dt asTime = dta last.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	s _ Time totalSeconds.
	dt _ DateAndTime fromSeconds: s.
	dta _ Time dateAndTimeFromSeconds: s.
	self assert: dt asDate = dta first.
	self assert: dt asTime = dta last
! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'dtl 11/14/2004 18:31'!
testTimeAsDateAndTime

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatsbase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testTimeAsDateAndTime) debug"

	| dtNow ptNow timeNow |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	timeNow _ Time dateAndTimeNow.
	dtNow _ DateAndTime now.
	ptNow _ PointInTime now asDateAndTime.
	self assert: (dtNow - ptNow) asSeconds abs < 1.
	self assert: ptNow asDate = timeNow first.
	self assert: (ptNow asTime asSeconds - timeNow last asSeconds) abs < 1.
	self assert: dtNow asDate = timeNow first.
	self assert: (dtNow asTime asSeconds - timeNow last asSeconds) abs < 1.
	
	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	timeNow _ Time dateAndTimeNow.
	dtNow _ DateAndTime now.
	ptNow _ PointInTime now asDateAndTime.
	self assert: (dtNow - ptNow) asSeconds abs < 1.
	self assert: ptNow asDate = timeNow first.
	self assert: (ptNow asTime asSeconds - timeNow last asSeconds) abs < 1.
	self assert: dtNow asDate = timeNow first.
	self assert: (dtNow asTime asSeconds - timeNow last asSeconds) abs < 1.
! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'dtl 11/14/2004 18:30'!
testTimeNow
	"Make sure DateAndTime>>now and Time>>now show the same time. Allow
	one second of variance in the test to account for difference in precision
	of DateAndTime seconds and the integer seconds clock."

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatsbase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testTimeNow) debug"

	| dt t dtSecs tSecs diff |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	dt _ DateAndTime now.
	t _ Time now.
	dtSecs _ (dt hour * 3600) + (dt minute * 60) + dt second.
	tSecs _ (t hour * 3600) + (t minute * 60) + t second.
	diff _ (dtSecs - tSecs) abs.
	self assert: diff < 1.
	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	dt _ DateAndTime now.
	t _ Time now.
	dtSecs _ (dt hour * 3600) + (dt minute * 60) + dt second.
	tSecs _ (t hour * 3600) + (t minute * 60) + t second.
	diff _ (dtSecs - tSecs) abs.
	self assert: diff < 1! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'dtl 11/21/2004 12:55'!
testTimeNowSeconds
	"Check consistency of various views on the current time"

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatsbase calculates
	Posix seconds from local seconds using the time zone offset. This test may
	also fail intermittently if it is interrupted by the process scheduler between
	calls to obtain the current time; these failures may be disregarded."

	"(self new setTestSelector: #testTimeNowSeconds) debug"

	| clockSeconds dtArray dt offset dateSeconds timeSeconds dtArraySeconds pt ptSeconds delta |
	delta _ 1.5. "Number of seconds of error to allow to accomodate scheduler jitter"
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	clockSeconds _ Time totalSeconds.
	dtArray _ Time dateAndTimeNow.
	dt _ DateAndTime now.
	pt _ PointInTime now.
	offset _ LocalTimeTransform here localOffsetSecondsAt: PointInTime now.
	dateSeconds _ dtArray first asSeconds + offset. "local squeak seconds clock"
	timeSeconds _ dtArray last asSeconds.
	dtArraySeconds _ (dateSeconds + timeSeconds - offset).
	ptSeconds _ pt asLocalSmalltalkSeconds.
	self assert: (clockSeconds - dt asSeconds) abs < delta.
	self assert: (clockSeconds - dtArraySeconds) abs < delta.
	self assert: (clockSeconds - ptSeconds) abs < delta.
	self assert: clockSeconds = (DateAndTime fromSeconds: clockSeconds) asSeconds.
	self assert: clockSeconds = (PointInTime
		fromSmalltalkSeconds: clockSeconds) asLocalSmalltalkSeconds.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	clockSeconds _ Time totalSeconds.
	dtArray _ Time dateAndTimeNow.
	dt _ DateAndTime now.
	pt _ PointInTime now.
	offset _ LocalTimeTransform here localOffsetSecondsAt: PointInTime now.
	dateSeconds _ dtArray first asSeconds + offset. "local squeak seconds clock"
	timeSeconds _ dtArray last asSeconds.
	dtArraySeconds _ (dateSeconds + timeSeconds - offset).
	ptSeconds _ pt asLocalSmalltalkSeconds.
	self assert: (clockSeconds - dt asSeconds) abs < delta.
	self assert: (clockSeconds - dtArraySeconds) abs < delta.
	self assert: (clockSeconds - ptSeconds) abs < delta.
	self assert: clockSeconds = (DateAndTime fromSeconds: clockSeconds) asSeconds.
	self assert: clockSeconds = (PointInTime
		fromSmalltalkSeconds: clockSeconds) asLocalSmalltalkSeconds.
! !

!TimeTransformTest methodsFor: 'testing-Duration' stamp: 'dtl 11/2/2004 15:46'!
testDurationForSamePointInTimeIsZero
	"The same PointInTime expressed as DateInTime for two different time zones should have
	a Duration difference of zero."

	"(self new setTestSelector: #testDurationForSamePointInTimeIsZero) debug"

	| aPointInTime est mdt estDateAndTime mdtDateAndTime duration |
	aPointInTime _ PointInTime fromAbsoluteSeconds: 1100000000.
	est _ TimeZoneDatabase systemDatabase timeZoneFor: 'posix/America/Detroit'.
	mdt _ TimeZoneDatabase systemDatabase timeZoneFor: 'posix/America/Phoenix'.
	estDateAndTime _ aPointInTime asDateAndTimeForTimeZone: est.
	mdtDateAndTime _ aPointInTime asDateAndTimeForTimeZone: mdt.
	self assert: (estDateAndTime asPointInTime asAbsoluteSeconds = mdtDateAndTime asPointInTime asAbsoluteSeconds).
	duration _ mdtDateAndTime - estDateAndTime.
	self assert: duration asSeconds = 0.

! !

!TimeTransformTest methodsFor: 'testing-Duration' stamp: 'dtl 11/2/2004 14:21'!
testDurationWithNoTransition
	"Just a normal duration, no DST or leap seconds to worry about. This tests the DateAndTime>>- method."

	"(self new setTestSelector: #testDurationWithNoTransition) debug"

	| ptMidnight ptAfterDstTransition dtMidnight dtAfterDstTransition duration secondsDifference durationSeconds anArbitrarySecondsCount |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	anArbitrarySecondsCount _ 1100000000.
	ptMidnight _ PointInTime fromAbsoluteSeconds: anArbitrarySecondsCount - 1.
	ptAfterDstTransition _ PointInTime fromAbsoluteSeconds: anArbitrarySecondsCount + 1.
	secondsDifference _ ptAfterDstTransition asAbsoluteSeconds - ptMidnight asAbsoluteSeconds.
	self assert: secondsDifference = 2.

	dtMidnight _ ptMidnight asDateAndTime.
	dtAfterDstTransition _ ptAfterDstTransition asDateAndTime.
	duration _ dtAfterDstTransition - dtMidnight.
	durationSeconds _ duration asSeconds.
	self assert: durationSeconds = secondsDifference.

! !

!TimeTransformTest methodsFor: 'testing-Duration' stamp: 'dtl 11/6/2004 19:37'!
testTwoDateAndTimesTheSame

	"(self new setTestSelector: #testTwoDateAndTimesTheSame) debug"

	| dtOne dtTwo duration ptOne ptTwo |
	"At the same real time, the wall clock shows one hour less as we move westward by one hour of Earth angle."

	dtOne _ '2004-11-27T22:00:00-04:00' asDateAndTime.
	dtTwo _ '2004-11-27T21:00:00-05:00' asDateAndTime.

	duration _ dtTwo - dtOne.
	self assert: duration asSeconds = 0.

	"Verify that they are the same point in time."
	ptOne _ dtOne asPointInTime.
	ptTwo _ dtTwo asPointInTime.
	self assert: ptOne asAbsoluteSeconds = ptTwo asAbsoluteSeconds
! !

!TimeTransformTest methodsFor: 'testing-Duration' stamp: 'dtl 11/6/2004 22:33'!
testTwoDateAndTimesWithDifferentOffset
	"Two DateAndTimes with the same wall clock display, but with different time zone
	offsets. The one with the larger offset corresponds to a more westerly location
	on the planet, and thus represents a PointInTime of greater magnitude."

	"(self new setTestSelector: #testTwoDateAndTimesWithDifferentOffset) debug"

	| dtOne dtTwo duration ptOne ptTwo |
	"dtTwo is the same wall clock time, but one hour to the west, hence is a time that occurs
	one hour later than dtOne."
	dtOne _ '2004-11-27T22:00:00-04:00' asDateAndTime.
	dtTwo _ '2004-11-27T22:00:00-05:00' asDateAndTime.

	ptOne _ dtOne asPointInTime.
	ptTwo _ dtTwo asPointInTime.
	self assert: (ptTwo asAbsoluteSeconds - ptOne asAbsoluteSeconds) = 3600.

	duration _ dtTwo - dtOne.
	self assert: duration asSeconds = 3600.
! !

!TimeTransformTest methodsFor: 'testing-Duration DST' stamp: 'dtl 11/7/2004 21:45'!
testDurationAcrossDstTransition
	"Test DateAndTime>>- across a DST transition."

	"(self new setTestSelector: #testDurationAcrossDstTransition) debug"

	| ptMidnight ptAfterDstTransition dtMidnight dtAfterDstTransition duration secondsDifference durationSeconds |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	ptMidnight _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (3600 * 2).
	ptAfterDstTransition _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (3600 * 6).
	secondsDifference _ ptAfterDstTransition asAbsoluteSeconds - ptMidnight asAbsoluteSeconds.
	self assert: secondsDifference = (3600 * 8).

	dtMidnight _ ptMidnight asDateAndTime.
	dtAfterDstTransition _ ptAfterDstTransition asDateAndTime.

	self assert: (dtAfterDstTransition asPointInTime asPosixSeconds - dtMidnight asPointInTime asPosixSeconds) = (3600 * 8).

	duration _ dtAfterDstTransition - dtMidnight.
	durationSeconds _ duration asSeconds.
	self assert: durationSeconds = secondsDifference.

! !

!TimeTransformTest methodsFor: 'testing-Duration DST' stamp: 'dtl 11/2/2004 13:52'!
testSecondsCountAcrossDstTransition

	"(self new setTestSelector: #testSecondsCountAcrossDstTransition) debug"

	| ptMidnight ptAfterDstTransition secondsDifference localSecsMidnight localSecsAfterDstTransition |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	ptMidnight _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (3600 * 2).
	ptAfterDstTransition _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (3600 * 6).
	secondsDifference _ ptAfterDstTransition asAbsoluteSeconds - ptMidnight asAbsoluteSeconds.
	self assert: secondsDifference = (3600 * 8).

	localSecsMidnight _ ptMidnight asLocalSmalltalkSeconds.
	localSecsAfterDstTransition _ ptAfterDstTransition asLocalSmalltalkSeconds.
	self assert: (localSecsAfterDstTransition - localSecsMidnight) = (secondsDifference - 3600).
! !

!TimeTransformTest methodsFor: 'testing-Duration DST' stamp: 'dtl 11/2/2004 13:54'!
testSecondsCountAcrossDstTransitionNoLeapSeconds

	"(self new setTestSelector: #testSecondsCountAcrossDstTransitionNoLeapSeconds) debug"

	| ptMidnight ptAfterDstTransition secondsDifference localSecsMidnight localSecsAfterDstTransition |
	TimeZoneDatabase systemDatabase defaultLocation: 'posix/America/Detroit'.
	ptMidnight _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (3600 * 2).
	ptAfterDstTransition _ PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (3600 * 6).
	secondsDifference _ ptAfterDstTransition asAbsoluteSeconds - ptMidnight asAbsoluteSeconds.
	self assert: secondsDifference = (3600 * 8).

	localSecsMidnight _ ptMidnight asLocalSmalltalkSeconds.
	localSecsAfterDstTransition _ ptAfterDstTransition asLocalSmalltalkSeconds.
	self assert: (localSecsAfterDstTransition - localSecsMidnight) = (secondsDifference - 3600).
! !

!TimeTransformTest methodsFor: 'testing-Duration leap seconds' stamp: 'dtl 11/2/2004 14:06'!
testSecondsCountAcrossLeapSecondTransition
	"PointInTime seconds are atomic clock seconds. Posix seconds and Smalltalk
	seconds are both rotational seconds. Leap seconds are added so that rotational
	seconds counting stays aligned with the rotation of the Earth."

	"(self new setTestSelector: #testSecondsCountAcrossLeapSecondTransition) debug"

	| ptBefore ptAfter secondsDifference localSecsBefore localSecsAfter aLeapSecondTransitionTime posixSecsBefore posixSecsAfter |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	aLeapSecondTransitionTime _ 915148821. "From the leap second table in right/America/Detroit"

	"Set up an absolute time duration of two seconds, spanning the leap second transition"
	ptBefore _ PointInTime fromAbsoluteSeconds: aLeapSecondTransitionTime - 1.
	ptAfter _ PointInTime fromAbsoluteSeconds: aLeapSecondTransitionTime + 1.
	secondsDifference _ ptAfter asAbsoluteSeconds - ptBefore asAbsoluteSeconds.
	self assert: secondsDifference = 2.

	"Local seconds should count two absolute seconds as the same local second"
	localSecsBefore _ ptBefore asLocalSmalltalkSeconds.
	localSecsAfter _ ptAfter asLocalSmalltalkSeconds.
	self assert: (localSecsAfter - localSecsBefore) = 1.

	"Posix seconds also count two absolute seconds as the same Posix second"
	posixSecsBefore _ ptBefore asPosixSeconds.
	posixSecsAfter _ ptAfter asPosixSeconds.
	self assert: (posixSecsAfter - posixSecsBefore) = 1.


! !

!TimeTransformTest methodsFor: 'testing-arithmetic' stamp: 'dtl 11/12/2004 09:57'!
testAddDurationAcrossDstTransition

	"(self new setTestSelector: #testAddDurationAcrossDstTransition) debug"

	| midnight fourHoursLater duration |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	midnight _ '2004-10-31T00:00:00-04:00' asDateAndTime.
	fourHoursLater _ '2004-10-31T03:00:00-05:00' asDateAndTime.
	duration _ Duration seconds: (4 * 3600).
	self assert: midnight + duration = fourHoursLater
! !

!TimeTransformTest methodsFor: 'testing-arithmetic' stamp: 'dtl 11/12/2004 10:08'!
testAddDurationAcrossLeapSecondTransition

	"(self new setTestSelector: #testAddDurationAcrossLeapSecondTransition) debug"

	| aLeapSecondTransitionTime ptFirst ptTenSecondsLater |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	aLeapSecondTransitionTime _ 915148821. "From the leap second table in right/America/Detroit"
	ptFirst _ PointInTime fromAbsoluteSeconds: aLeapSecondTransitionTime - 2.
	ptTenSecondsLater _ PointInTime fromAbsoluteSeconds: aLeapSecondTransitionTime - 2 + 10.
	self assert: (ptTenSecondsLater asAbsoluteSeconds - ptFirst asAbsoluteSeconds) = 10.
	self assert: (ptTenSecondsLater asPosixSeconds - ptFirst asPosixSeconds) = 9.
	self assert: (ptTenSecondsLater asLocalSmalltalkSeconds - ptFirst asLocalSmalltalkSeconds) = 9.
	self assert: (ptFirst asDateAndTime + (Duration seconds: 9)) = ptTenSecondsLater
! !

!TimeTransformTest methodsFor: 'testing-arithmetic' stamp: 'dtl 11/6/2004 22:32'!
testSubtractDateAndTime
	"Test DateAndTime>>- "

	"(self new setTestSelector: #testSubtractDateAndTime) debug"

	| dtOne dtTwo twoMinusOne dtThree threeMinusOne dtFour dtFive fiveMinusFour dtSix dtSeven sevenMinusSix |
	"One day duration"
	dtOne _ '2004-11-27T23:59:37-05:00' asDateAndTime.
	dtTwo _ '2004-11-28T23:59:37-05:00' asDateAndTime.
	twoMinusOne _ dtTwo - dtOne.
	self assert: twoMinusOne asSeconds = (24 * 3600).

	"dtThree is the same wall clock display, but one hour to the east, hence it represents one
	hour earlier in absolute time."
	dtThree _ '2004-11-27T23:59:37-04:00' asDateAndTime.
	threeMinusOne _ dtThree - dtOne.
	self assert: threeMinusOne asSeconds = -3600.

	"At the same real time, the wall clock shows one hour less as we move westward by one hour of Earth angle."
	dtFour _ '2004-11-27T23:59:37-04:00' asDateAndTime.
	dtFive _ '2004-11-27T22:59:37-05:00' asDateAndTime.
	fiveMinusFour _ dtFive - dtFour.
	self assert: fiveMinusFour asSeconds = 0.

	dtSix _ '2004-11-27T22:59:37-04:00' asDateAndTime.
	dtSeven _ '2004-11-27T23:59:37-05:00' asDateAndTime.
	sevenMinusSix _ dtSeven - dtSix.
	self assert: sevenMinusSix asSeconds = (3600 * 2).
! !

!TimeTransformTest methodsFor: 'testing-arithmetic' stamp: 'dtl 11/12/2004 09:52'!
testSubtractDateAndTimeAcrossDstTransition

	"(self new setTestSelector: #testSubtractDateAndTimeAcrossDstTransition) debug"

	| ptMidnight ptMidnightFourHoursLater midnight fourHoursLater difference |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	ptMidnight _ PointInTime
		fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (2 * 3600).
	ptMidnightFourHoursLater _ PointInTime
		fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (2 * 3600).
	midnight _ '2004-10-31T00:00:00-04:00' asDateAndTime.
	fourHoursLater _ '2004-10-31T03:00:00-05:00' asDateAndTime.
	self assert: (ptMidnight asDateAndTime - midnight) asSeconds abs < 1.
	self assert: (ptMidnightFourHoursLater asDateAndTime - fourHoursLater) asSeconds abs < 1.
	difference _ fourHoursLater - midnight.
	self assert: 4 * 3600 = difference asSeconds
! !

!TimeTransformTest methodsFor: 'testing-arithmetic' stamp: 'dtl 11/12/2004 10:09'!
testSubtractDurationAcrossDstTransition

	"(self new setTestSelector: #testSubtractDurationAcrossDstTransition) debug"

	| midnight fourHoursLater duration |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	midnight _ '2004-10-31T00:00:00-04:00' asDateAndTime.
	fourHoursLater _ '2004-10-31T03:00:00-05:00' asDateAndTime.
	duration _ Duration seconds: (4 * 3600).
	self assert: fourHoursLater - duration = midnight
! !

!TimeTransformTest methodsFor: 'testing-arithmetic' stamp: 'dtl 11/12/2004 10:09'!
testSubtractDurationAcrossLeapSecondTransition

	"(self new setTestSelector: #testSubtractDurationAcrossLeapSecondTransition) debug"

	| aLeapSecondTransitionTime ptFirst ptTenSecondsLater |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	aLeapSecondTransitionTime _ 915148821. "From the leap second table in right/America/Detroit"
	ptFirst _ PointInTime fromAbsoluteSeconds: aLeapSecondTransitionTime - 2.
	ptTenSecondsLater _ PointInTime fromAbsoluteSeconds: aLeapSecondTransitionTime - 2 + 10.
	self assert: (ptTenSecondsLater asAbsoluteSeconds - ptFirst asAbsoluteSeconds) = 10.
	self assert: (ptTenSecondsLater asPosixSeconds - ptFirst asPosixSeconds) = 9.
	self assert: (ptTenSecondsLater asLocalSmalltalkSeconds - ptFirst asLocalSmalltalkSeconds) = 9.
	self assert: (ptTenSecondsLater asDateAndTime - (Duration seconds: 9)) = ptFirst
! !

!TimeTransformTest methodsFor: 'testing-time plugin' stamp: 'dtl 11/23/2004 21:43'!
testPrimPosixTimeMicrosecondResolution
	"Make sure that local time is reported similarly with and without the plugin.
	The #assert: is only evaluated if the plugin is available, otherwise this test
	skips the #assert: passes."

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatsbase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testPrimPosixTimeMicrosecondResolution) debug"

	| primSeconds expectedSeconds |

	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	primSeconds _ PointInTime primPosixTimeMicrosecondResolution.
	expectedSeconds _ Time totalSeconds - PointInTime posixOffset
		- (LocalTimeTransform here localOffsetSecondsAt: Time totalSeconds).
	primSeconds ifNotNil:
		[self assert: (primSeconds - expectedSeconds) abs < 1].

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	primSeconds _ PointInTime primPosixTimeMicrosecondResolution.
	expectedSeconds _ Time totalSeconds - PointInTime posixOffset
		- (LocalTimeTransform here localOffsetSecondsAt: Time totalSeconds).
	primSeconds ifNotNil:
		[self assert: (primSeconds - expectedSeconds) abs < 1]
! !

!TimeTransformTest methodsFor: 'testing-time plugin' stamp: 'dtl 11/17/2004 19:52'!
testPrimPosixTimeMicrosecondResolutionOffset

	"(self new setTestSelector: #testPrimPosixTimeMicrosecondResolutionOffset) debug"

	| primSecs offset smalltalkSeconds |
	primSecs _ PointInTime primPosixTimeMicrosecondResolution.
	primSecs ifNotNil:
		[offset _ PointInTime primPosixTimeAndLocalOffset second.
		smalltalkSeconds _ Time totalSeconds.
		self assert: (primSecs - (smalltalkSeconds - PointInTime posixOffset + offset)) abs < 1]
! !

!TimeTransformTest methodsFor: 'testing-time plugin' stamp: 'dtl 11/17/2004 20:05'!
testPrimSecondsClock
	"Compare the default Squeak clock prim seconds to what is reported by
	the plugin."

	"(self new setTestSelector: #testPrimSecondsClock) debug"

	| primSecondsClock primPosixTime secondsFromPosixTime diff offset |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	primSecondsClock _ self primSecondsClock.
	(primPosixTime _ PointInTime primPosixTimeMicrosecondResolution) ifNotNil:
		[secondsFromPosixTime _ primPosixTime + PointInTime posixOffset.
		offset _ self primPosixTimeAndLocalOffset last.
		diff _ secondsFromPosixTime - primSecondsClock - offset.
		self assert: diff abs < 1]
! !

!TimeTransformTest methodsFor: 'testing-time plugin' stamp: 'dtl 11/21/2004 11:23'!
testPrimVersusApproximateTimeNow
	"Compare the default Squeak clock prim seconds to what is reported by
	the plugin."

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatsbase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testPrimVersusApproximateTimeNow) debug"

	| approxTime primPosixTime diff |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	approxTime _ Time totalSeconds - PointInTime posixOffset
		- (LocalTimeTransform here localOffsetSecondsAt: Time totalSeconds).
	(primPosixTime _ PointInTime primPosixTimeMicrosecondResolution) ifNotNil:
		[diff _ primPosixTime - approxTime.
		self assert: diff abs < 1].

	TimeZoneDatabase systemDatabase defaultLocation: 'posix/America/Detroit'.
	approxTime _ Time totalSeconds - PointInTime posixOffset
		- (LocalTimeTransform here localOffsetSecondsAt: Time totalSeconds).
	(primPosixTime _ PointInTime primPosixTimeMicrosecondResolution) ifNotNil:
		[diff _ primPosixTime - approxTime.
		self assert: diff abs < 1].

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	approxTime _ Time totalSeconds - PointInTime posixOffset
		- (LocalTimeTransform here localOffsetSecondsAt: Time totalSeconds).
	(primPosixTime _ PointInTime primPosixTimeMicrosecondResolution) ifNotNil:
		[diff _ primPosixTime - approxTime.
		self assert: diff abs < 1]
! !

!TimeTransformTest methodsFor: 'testing-misc' stamp: 'dtl 11/7/2004 17:22'!
testLocalDetroitEpoch
	"Check DateAndTime for the beginning of the Posix epoch expressed as time in
	the local time zone (see #setUp)."

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	p _	(PointInTime fromPosixSeconds: 0) asDateAndTime.
	self assert: p julianDayNumber = 2440587.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p year = 1969.
	self assert: p month = 12.
	self assert: p hour = 19.
	self assert: p minute = 0.
	self assert: p second = 0
! !

!TimeTransformTest methodsFor: 'testing-misc' stamp: 'dtl 10/11/2004 22:33'!
testPosixEpoch
	"Check DateAndTime for the beginning of the Posix epoch expressed as UTC."

	| p |
	p _	(PointInTime fromPosixSeconds: 0)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'UTC').
	self assert: p seconds == 0.
	self assert: p julianDayNumber = 2440588.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: 0)
! !

!TimeTransformTest methodsFor: 'testing-misc' stamp: 'dtl 10/30/2004 22:47'!
testTimeZoneTransitionSecond
	"This test just makes sure that the time zone tables contain the expected
	transition seconds. If the test fails, then either the tables do not contain
	the expected transition times, or there is something wrong with the offset
	lookup. Neither of these things should ever happen, so you'll have to
	figure it out on your own ;-)"

	"self new testTimeZoneTransitionSecond"

	TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit'.
	self assert: self absoluteDstTransition = 1099202422.
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	self assert: self posixDstTransition = 1099202400.

	TimeZoneDatabase systemDatabase defaultLocation: 'posix/America/Detroit'.
	self assert: self absoluteDstTransition = 1099202400.
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	self assert: self posixDstTransition = 1099202400.
! !

!TimeTransformTest methodsFor: 'private' stamp: 'dtl 10/23/2004 16:01'!
absoluteDstTransition
	"The first UTC second for the next daylight savings transition in absolute seconds.
	Search the actual time zone table, just in case someone changes the table retroactively."

	"self new absoluteDstTransition"

	| secs tz summerOffset dayAfterTransition hourAfterTransition secondAfterTransition minuteAfterTransition |
	secs _ 1096831000. "A time during October 2004"
	tz _ LocalTimeTransform here.
	summerOffset _ tz localOffsetSecondsAt: (PointInTime fromAbsoluteSeconds: secs).
	dayAfterTransition _ (secs to: secs + 3600 * 24 * 100 by: 3600 * 24)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromAbsoluteSeconds: s))
				== summerOffset) not].
	hourAfterTransition _ (dayAfterTransition - (3600 * 24) to: dayAfterTransition by: 3600)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromAbsoluteSeconds: s))
				== summerOffset) not].
	minuteAfterTransition _ (hourAfterTransition - 3600 to: hourAfterTransition by: 60)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromAbsoluteSeconds: s))
				== summerOffset) not].
	secondAfterTransition _ (minuteAfterTransition - 60 to: minuteAfterTransition by: 1)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromAbsoluteSeconds: s))
				== summerOffset) not].
	^ secondAfterTransition
! !

!TimeTransformTest methodsFor: 'private' stamp: 'dtl 10/23/2004 17:08'!
localSmalltalkDstTransition
	"The first local Smalltalk second for the next daylight savings transition in
	the local time zone."

	"self new localSmalltalkDstTransition"

	| secs tz summerOffset dayAfterTransition hourAfterTransition secondAfterTransition minuteAfterTransition |
	secs _ 1096831000 + self posixOffset. "A time during October 2004"
	tz _ LocalTimeTransform here.
	summerOffset _ tz localOffsetSecondsAt: (PointInTime fromSeconds: secs).
	dayAfterTransition _ (secs to: secs + 3600 * 24 * 100 by: 3600 * 24)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromSeconds: s))
				== summerOffset) not].
	hourAfterTransition _ (dayAfterTransition - (3600 * 24) to: dayAfterTransition by: 3600)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromSeconds: s))
				== summerOffset) not].
	minuteAfterTransition _ (hourAfterTransition - 3600 to: hourAfterTransition by: 60)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromSeconds: s))
				== summerOffset) not].
	secondAfterTransition _ (minuteAfterTransition - 60 to: minuteAfterTransition by: 1)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromSeconds: s))
				== summerOffset) not].
	^ secondAfterTransition
! !

!TimeTransformTest methodsFor: 'private' stamp: 'dtl 10/3/2004 17:18'!
posixDstTransition
	"The first UTC Posix second for the next daylight savings transition. Search the
	actual time zone table, just in case someone changes the table retroactively."

	"self new posixDstTransition"

	| secs tz summerOffset dayAfterTransition hourAfterTransition secondAfterTransition minuteAfterTransition |
	secs _ 1096831000. "A time during October 2004"
	tz _ LocalTimeTransform here.
	summerOffset _ tz localOffsetSecondsAt: (PointInTime fromPosixSeconds: secs).
	dayAfterTransition _ (secs to: secs + 3600 * 24 * 100 by: 3600 * 24)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromPosixSeconds: s))
				== summerOffset) not].
	hourAfterTransition _ (dayAfterTransition - (3600 * 24) to: dayAfterTransition by: 3600)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromPosixSeconds: s))
				== summerOffset) not].
	minuteAfterTransition _ (hourAfterTransition - 3600 to: hourAfterTransition by: 60)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromPosixSeconds: s))
				== summerOffset) not].
	secondAfterTransition _ (minuteAfterTransition - 60 to: minuteAfterTransition by: 1)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromPosixSeconds: s))
				== summerOffset) not].
	^ secondAfterTransition
! !

!TimeTransformTest methodsFor: 'private' stamp: 'dtl 10/23/2004 12:59'!
posixOffset
	"Difference in seconds between Smalltalk epoch and Posix epoch."
	"52 * 365 + (17 * 366) * 24 * 60 * 60"

	^ 2177452800! !

!TimeTransformTest methodsFor: 'private' stamp: 'dtl 11/14/2004 17:22'!
primPosixTimeAndLocalOffset
	"Array of two integers, with number of seconds since since Jan 1, 1970 UTC,
	and the local offset for the time zone in effect for this system."

	<primitive: 'primitivePosixTimeAndLocalOffset' module: 'TimeFunctions'>
	^ nil! !

!TimeTransformTest methodsFor: 'private' stamp: 'dtl 11/14/2004 17:15'!
primSecondsClock
	"Answer the number of seconds since 00:00 on the morning of
	 January 1, 1901 (a 32-bit unsigned number).
	 Essential. See Object documentation whatIsAPrimitive. "

	<primitive: 137>
	self primitiveFailed! !

!TimeTransformTest methodsFor: 'private' stamp: 'dtl 11/23/2004 21:14'!
setDstTransitionWatcherFall2004

	"Set the DST transition watcher to think that the last DST transition
	happened in the Fall of 2004. This will permit tests that rely on the DST
	transition watcher to work properly in the future. This method must be
	called after the time zone setting is changed, because the DST transition
	watcher is automatically reset whenever such a change occurs."

	| lateFall2004 newLastTransitionTick |
	lateFall2004 _ PointInTime fromAbsoluteSeconds: 1100000000.
	newLastTransitionTick _ TimeZoneDstTransitionWatcher
		clockWatcher probableLastTransitionTickBefore: lateFall2004.
	TimeZoneDstTransitionWatcher clockWatcher transitionTick: newLastTransitionTick.
! !

!TimeTransformTest methodsFor: 'private' stamp: 'dtl 11/23/2004 21:14'!
setDstTransitionWatcherSpring2004

	"Set the DST transition watcher to think that the last DST transition
	happened in the Spring of 2004. This will permit tests that rely on the DST
	transition watcher to work properly in the future. This method must be
	called after the time zone setting is changed, because the DST transition
	watcher is automatically reset whenever such a change occurs."

	| earlySummer2004 newLastTransitionTick |
	earlySummer2004 _ PointInTime fromAbsoluteSeconds: 1084440000.
	newLastTransitionTick _ TimeZoneDstTransitionWatcher
		clockWatcher probableLastTransitionTickBefore: earlySummer2004.
	TimeZoneDstTransitionWatcher clockWatcher transitionTick: newLastTransitionTick.
! !


!TimeZone methodsFor: '*time-TimeZones' stamp: 'dtl 11/13/2004 12:18'!
dstTransitionTimes

	^ #()! !

!TimeZone methodsFor: '*time-TimeZones' stamp: 'dtl 11/8/2004 21:23'!
leapSecondsAt: aPointInTime
	"Ignore leap seconds"

	^ 0
! !

!TimeZone methodsFor: '*time-TimeZones' stamp: 'dtl 11/12/2004 10:31'!
localOffsetSecondsAt: aPointInTime

	^ self offset asSeconds! !


!TimeZoneDatabase methodsFor: 'accessing' stamp: 'dtl 11/13/2004 13:50'!
defaultLocation: aString

	"TimeZoneDatabase systemDatabase defaultLocation: 'right/America/Detroit' "

	defaultLocation _ aString.
	defaultTimeZone _ nil.
	self changed: #timeZoneDatabaseDefaultZone
! !

!TimeZoneDatabase methodsFor: 'accessing' stamp: 'dtl 11/9/2004 06:31'!
defaultTimeZone

	"TimeZoneDatabase systemDatabase defaultTimeZone"

	^ defaultTimeZone ifNil: [defaultTimeZone _ self timeZoneFor: self defaultLocation]! !

!TimeZoneDatabase methodsFor: 'querying' stamp: 'dtl 9/13/2004 11:36'!
hoursFrom: name to: anotherTimeZoneName
	"Ask the database for the offset in hours between two time zones. The result
	may be an integer or a fraction."

	"TimeZoneDatabase systemDatabase hoursFrom: 'Europe/Amsterdam' to: 'America/Detroit'"
	"(TimeZoneDatabase systemDatabase hoursFrom: 'CET' to: 'Canada/Newfoundland') asFloat"

	^ (self offsetFrom: name to: anotherTimeZoneName) / 3600
! !

!TimeZoneDatabase methodsFor: 'querying' stamp: 'dtl 9/20/2004 19:32'!
offsetFrom: name to: anotherTimeZoneName
	"Ask the database for the current offset in seconds between two time zones."

	"TimeZoneDatabase systemDatabase offsetFrom: 'Europe/Amsterdam' to: 'America/Detroit'"

	| now |
	now _ PointInTime now.
	^ (self offsetFor: name at: now) - (self offsetFor: anotherTimeZoneName at: now)
! !

!TimeZoneDatabase methodsFor: 'querying' stamp: 'dtl 11/9/2004 06:48'!
timeZoneName
	"Answer the name of the active local time transform"

	^ self defaultLocation asString
! !

!TimeZoneDatabase methodsFor: 'querying' stamp: 'dtl 11/14/2004 13:11'!
tzNameAbbreviations
	"Answer the abbreviations for the active local time transform"

	| zone |
	zone _ self defaultTimeZone.
	^ zone isNil
		ifTrue: [#()]
		ifFalse: [zone tzNameAbbreviations]
! !

!TimeZoneDatabase methodsFor: 'offsets from UTC' stamp: 'dtl 11/13/2004 15:05'!
localOffsetSecondsAt: aPointInTimeOrPosixSeconds
	"Answer the offset seconds for a point in time (or its integer value) for
	the active local time transform."

	^ self defaultTimeZone isNil
		ifTrue: [0]
		ifFalse: [defaultTimeZone localOffsetSecondsAt: aPointInTimeOrPosixSeconds]
! !

!TimeZoneDatabase methodsFor: 'leap seconds' stamp: 'dtl 11/13/2004 15:04'!
leapSecondsAt: aPointInTimeOrPosixSeconds
	"Answer the leap seconds for a point in time (or its integer value) for
	the active local time transform."

	^ self defaultTimeZone isNil
		ifTrue: [0]
		ifFalse: [defaultTimeZone leapSecondsAt: aPointInTimeOrPosixSeconds]! !

!TimeZoneDatabase methodsFor: 'time zone proxy' stamp: 'dtl 11/14/2004 13:13'!
abbreviation
	"Answer one of possibly several abbreviations for the current time zone setting"

	| abbreviations |
	abbreviations _ self tzNameAbbreviations.
	^ abbreviations isEmpty
		ifTrue: []
		ifFalse: [abbreviations anyOne asString]
! !

!TimeZoneDatabase methodsFor: 'time zone proxy' stamp: 'dtl 11/13/2004 15:06'!
dstTransitionTimes

	^ self defaultTimeZone isNil
		ifTrue: [#()]
		ifFalse: [defaultTimeZone dstTransitionTimes]! !

!TimeZoneDatabase methodsFor: 'time zone proxy' stamp: 'dtl 11/13/2004 22:43'!
name
	"The name of the active time zone, when the database is used as a TimeZone."

	^ (self defaultLocation ifNil: [super name]) asString! !

!TimeZoneDatabase methodsFor: 'time zone proxy' stamp: 'dtl 11/13/2004 22:45'!
offset

	^ Duration seconds: (self localOffsetSecondsAt: PointInTime now)! !

!TimeZoneDatabase methodsFor: '*kernel-Chronology' stamp: 'dtl 9/20/2004 19:32'!
durationFrom: name to: anotherTimeZoneName
	"Ask the database for the current offset between two time zones as a Duration."

	"TimeZoneDatabase systemDatabase durationFrom: 'Europe/Amsterdam' to: 'America/Detroit'"
	"TimeZoneDatabase systemDatabase durationFrom: 'CET' to: 'Canada/Newfoundland'"

	^ Duration seconds: (self offsetFrom: name to: anotherTimeZoneName)
! !


!TimeZoneDatabase class methodsFor: 'instance creation' stamp: 'dtl 11/13/2004 13:31'!
buildSystemDatabase
	"Build a full timezone database and install it as ThisSystemDatabase. Load the
	database from any tzfile files found in the default location for this system."

	"TimeZoneDatabase buildSystemDatabase"

	self thisSystemDatabase: (self fromZoneinfoDirectory: self tzPrefixPath).
	^ ThisSystemDatabase

! !

!TimeZoneDatabase class methodsFor: 'instance creation' stamp: 'dtl 9/13/2004 20:33'!
buildSystemDatabaseForSqueak
	"Build a full timezone database and install it as ThisSystemDatabase. Load the
	database from any tzfile files found in the default location for this system.
	Check first to determine if the time zone files exist, and issue a warning if
	the files are not available."

	"TimeZoneDatabase buildSystemDatabaseForSqueak"

	^ (FileDirectory on: self tzPrefixPath) entries isEmpty
		ifTrue: [self notify: 'no time zone files found in ', self tzPrefixPath]
		ifFalse: [self buildSystemDatabase]
! !

!TimeZoneDatabase class methodsFor: 'instance creation' stamp: 'dtl 11/13/2004 13:32'!
buildSystemDatabaseFromExamples
	"Build a full timezone database and install it as ThisSystemDatabase"

	"TimeZoneDatabase buildSystemDatabaseFromExamples"

	self thisSystemDatabase: (self fromExampleFiles).
	^ ThisSystemDatabase

! !

!TimeZoneDatabase class methodsFor: 'setting local time zone' stamp: 'dtl 11/9/2004 19:23'!
grepFor: aString
	"Convenience method to help find time zones for locations, especially if
	the name of the key city is known."

	"TimeZoneDatabase grepFor: 'Moscow' "
	"TimeZoneDatabase grepFor: 'Detroit' "
	"TimeZoneDatabase grepFor: 'UTC' "
	"TimeZoneDatabase grepFor: 'GMT' "

	^ self systemDatabase grepFor: aString
! !

!TimeZoneDatabase class methodsFor: 'setting local time zone' stamp: 'dtl 11/14/2004 12:35'!
setLocalTimeZone: tzName
	"To find a time zone for Detroit:
		TimeZoneDatabase grepFor: 'Detroit'
	Then use this method to set the desired time zone."

	"TimeZoneDatabase setLocalTimeZone: 'Europe/Moscow' "
	"TimeZoneDatabase setLocalTimeZone: 'right/America/Detroit' "
	"TimeZoneDatabase setLocalTimeZone: 'posix/America/Detroit' "
	"TimeZoneDatabase setLocalTimeZone: 'America/Detroit' "
	"TimeZoneDatabase setLocalTimeZone: 'UTC' "
	"TimeZoneDatabase setLocalTimeZone: 'GMT' "
	"TimeZoneDatabase setLocalTimeZone: 'Europe/London' "
	"TimeZoneDatabase setLocalTimeZone: 'Africa/Johannesburg' "
	"TimeZoneDatabase setLocalTimeZone: 'America/Los_Angeles' "

	self systemDatabase defaultLocation: tzName.
	^ self systemDatabase defaultTimeZone
! !

!TimeZoneDatabase class methodsFor: 'initialize-release' stamp: 'dtl 11/13/2004 13:50'!
thisSystemDatabase: anObject

	ThisSystemDatabase _ anObject.
	self changed: #timeZoneDatabaseForSystem! !

!TimeZoneDatabase class methodsFor: 'examples' stamp: 'dtl 11/23/2004 19:18'!
partyTimesForNewYear: newYear
	"Answer the times, in my own time zone, at which the New Year's Eve celebration
	for the newYear will occur for all time zones in the database. Sort the results so as to
	facilitate recognition of the grand events by committed revelers in my own time zone."

	"TimeZoneDatabase partyTimesForNewYear: 2005"

	| db allZones myTimeZone c idx dateAndTime seconds result |
	db _ TimeZoneDatabase systemDatabase.
	db isNil ifTrue: [
		self notify: 'no system time zone database'.
		^ nil].
	allZones _ db allTimeZones.
	myTimeZone _ LocalTimeTransform here.
	c _ SortedCollection sortBlock: [:x :y | (x at: 3) <= (y at: 3)].
	idx _ 0.
	self showProgressFrom: 1 to: allZones size withMessage: 'checking times'
		forBlock: [ :bar |
			allZones do: [:e |
			idx _ idx + 1.
			bar value: idx.
			dateAndTime _ e transformDate: (self dateFromString: '01.01.', newYear asString)
						time: (Time fromSeconds: 0)
						toTimeZone: myTimeZone.
			seconds _ (dateAndTime at: 1) asSeconds + (dateAndTime at: 2) asSeconds.
			c add: (Array
				with: e
				with: (dateAndTime)
				with: seconds)]].
	result _ OrderedCollection new.
	c do: [:e | result add: (Array with: (e at: 1) timeZoneName with: (e at: 2))].
	^ result asArray

! !


!TimeZoneDstTransitionWatcher methodsFor: 'initialize-release' stamp: 'dtl 11/13/2004 13:34'!
initialize

	DateAndTime addDependent: self.
	TimeZoneDatabase addDependent: self.
	TimeZoneDatabase systemDatabase addDependent: self.
	self start! !

!TimeZoneDstTransitionWatcher methodsFor: 'initialize-release' stamp: 'dtl 10/31/2004 09:07'!
start

	self stop.
	self monitorProcess:
		self monitorProcessBlock fork! !

!TimeZoneDstTransitionWatcher methodsFor: 'initialize-release' stamp: 'dtl 10/31/2004 09:06'!
stop

	self monitorProcess ifNotNilDo: [:p |
		p terminate.
		self monitorProcess: nil]! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 11/21/2000 18:59'!
accessProtect

	^ accessProtect ifNil: [accessProtect _ Semaphore forMutualExclusion]! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 10/31/2004 08:47'!
lastTick
	"Answer the value of lastTick"

	^ lastTick! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 10/31/2004 08:47'!
lastTick: anObject
	"Set the value of lastTick"

	lastTick _ anObject! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 10/31/2004 09:05'!
monitorProcess
	"Answer the value of monitorProcess"

	^ monitorProcess! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 10/31/2004 08:47'!
monitorProcess: anObject
	"Set the value of monitorProcess"

	monitorProcess _ anObject! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 10/31/2004 08:47'!
transitionTick
	"Answer the value of transitionTick"

	^ transitionTick! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 10/31/2004 08:47'!
transitionTick: anObject
	"Set the value of transitionTick"

	transitionTick _ anObject! !

!TimeZoneDstTransitionWatcher methodsFor: 'testing' stamp: 'dtl 11/22/2000 09:46'!
fallBackOccurredDuringHourBefore: aLocalSecondsCount
	"Answer true if a daylight savings time fall back transition has occured
	within the last hour prior to aLocalSecondsCount, which is assumed to
	represent the time now."

	"TimeZoneDstTransitionWatcher clockWatcher fallBackOccurredDuringHourBefore: Time totalSeconds"

	^ self transitionTick notNil and:
		[(aLocalSecondsCount - transitionTick) < 3600]
! !

!TimeZoneDstTransitionWatcher methodsFor: 'monitor' stamp: 'dtl 10/31/2004 09:12'!
isFallBack: secondsCount
	"True if secondsCount is between 0 and 3600 seconds less than the previously
	observed seconds clock tick, non-inclusive. This situation occurs when the
	clock is set back at a fall daylight savings time transition."

	| diff |
	self lastTick ifNil: [^ false].
	diff _ secondsCount - self lastTick.
	^ (diff < 0) and: [diff > -3600]! !

!TimeZoneDstTransitionWatcher methodsFor: 'monitor' stamp: 'dtl 11/23/2004 11:46'!
monitorProcessBlock
	"The monitor process needs to wake up only occasionally to keep the last
	tick reasonably up to date. PointInTime>>now calls #updateLastTick: with the
	current Smalltalk seconds clock in order to force an immediate update. This
	ensures that the DST monitor is up to date as of the call to PointInTime>>now,
	as required for a correct result at times very close to DST transition times."

	| delay |
	delay _ Delay forMilliseconds: 3000.
	^ [[self updateLastTick: Time totalSeconds.
		delay wait] repeat]! !

!TimeZoneDstTransitionWatcher methodsFor: 'monitor' stamp: 'dtl 11/22/2000 09:49'!
updateLastTick: currentLocalSmalltalkSeconds
	"Update the value of lastTick, and set transitionTick if a DST boundary
	has been crossed."

	self accessProtect critical:
		[(self isFallBack: currentLocalSmalltalkSeconds)
			ifTrue: [self transitionTick: currentLocalSmalltalkSeconds].
		self lastTick: currentLocalSmalltalkSeconds]
! !

!TimeZoneDstTransitionWatcher methodsFor: 'updating' stamp: 'dtl 11/13/2004 14:28'!
lastDstTransitionBefore: aPointInTime

	"self new lastDstTransitionBefore: PointInTime now"

	DateAndTime localTimeZone dstTransitionTimes
		inject: nil
		into: [:last :pt |
			(aPointInTime <= pt) ifTrue: [^ last].
			pt].
	^ nil
! !

!TimeZoneDstTransitionWatcher methodsFor: 'updating' stamp: 'dtl 11/13/2004 14:26'!
probableLastTransitionTick
	"If possible, answer the last Smalltalk seconds DST transition for the current
	time zone."

	"self new probableLastTransitionTick"

	^ self probableLastTransitionTickBefore: PointInTime now

! !

!TimeZoneDstTransitionWatcher methodsFor: 'updating' stamp: 'dtl 11/13/2004 14:25'!
probableLastTransitionTickBefore: aPointInTime
	"If possible, answer the last Smalltalk seconds DST transition before aPointInTime
	for the current time zone. This method permits unit tests to trick the DST transition
	watcher into working for a different time setting than the current system time."

	"self new probableLastTransitionTickBefore: PointInTime now"

	^ (self lastDstTransitionBefore: aPointInTime)
		ifNotNilDo: [:pt | pt asLocalSmalltalkSeconds]
! !

!TimeZoneDstTransitionWatcher methodsFor: 'updating' stamp: 'dtl 11/13/2004 14:03'!
update: aParameter
	"If the system time zone setting has changed, then the last recorded
	transition tick is no longer valid."

	aParameter == #timeZoneForDateAndTime
		ifTrue: [self transitionTick: self probableLastTransitionTick].
	aParameter == #timeZoneDatabaseDefaultZone
		ifTrue: [self transitionTick: self probableLastTransitionTick].
	aParameter == #timeZoneDatabaseForSystem
		ifTrue: [self transitionTick: self probableLastTransitionTick.
			TimeZoneDatabase systemDatabase addDependent: self].! !


!TimeZoneDstTransitionWatcher class methodsFor: 'class initialization' stamp: 'dtl 10/31/2004 09:36'!
initialize

	"self initialize"

	^ self startMonitoring! !

!TimeZoneDstTransitionWatcher class methodsFor: 'class initialization' stamp: 'dtl 10/31/2004 09:16'!
startMonitoring

	"self startMonitoring"

	^ self clockWatcher! !

!TimeZoneDstTransitionWatcher class methodsFor: 'class initialization' stamp: 'dtl 10/31/2004 09:17'!
stopMonitoring

	"self stopMonitoring"

	DstMonitor ifNotNilDo: [:m |
		m stop.
		DstMonitor _ nil.
		^ m].
	^ nil! !

!TimeZoneDstTransitionWatcher class methodsFor: 'instance creation' stamp: 'dtl 10/31/2004 09:10'!
clockWatcher

	"TimeZoneDstTransitionWatcher clockWatcher"

	^ DstMonitor ifNil: [DstMonitor _ self new]! !

!TimeZoneDstTransitionWatcher class methodsFor: 'tests' stamp: 'dtl 11/21/2000 22:02'!
verifyDstTransition
	"Log time info during the Fall 2004 DST transition. Set the operating system
	clock to 00:58am on Sunday October 31, 2004 (or some other convenient fall
	back transition). Then run this method (three hours run time).
	Look that the DST.log file to verify that the clock was set back as expected.
	Note that it is not possible to do this test without actually setting the operating
	system clock, since we want to ensure that Squeak is tracking the actual
	behavior of the operating system."

	"[self verifyDstTransition] forkAt: Processor userBackgroundPriority"

	| fileName delay f t dt str fallTransitionTime shortDelay endTime now |
	fileName _ 'DST.log'.
	fallTransitionTime _ 1099202422. "Fall 2004 in Detroit"

	"Match time zone to the OS time zone setting"
	TimeZoneDatabase setLocalTimeZone: 'right/America/Detroit'.

	delay _ Delay forMilliseconds: 9600. "Just under 10 seconds"
	shortDelay _ Delay forMilliseconds: 1000.
	[f _ FileStream fileNamed: fileName.
	"Wait for a 10 second boundary"
	[(Time totalSeconds \\ 10) = 0] whileFalse: [(Delay forMilliseconds: 100) wait].

	"Run for three hours (six times per minute)"
	endTime _ PointInTime now asAbsoluteSeconds + (3600 * 3).
	[(now _ PointInTime now asAbsoluteSeconds) < endTime] whileTrue:
		[t _ Time totalSeconds.
		dt _ DateAndTime now.
		str _ 'Squeak seconds: ', t printString, ', DateAndTime: ', dt printString.
		f nextPutAll: str; nextPut: Character lf; flush.
		((now - fallTransitionTime) abs < 21)
			ifTrue:
				[shortDelay wait]
			ifFalse:
				[delay wait.
				[(Time totalSeconds \\ 10) = 0] whileFalse: [(Delay forMilliseconds: 100) wait]]].
	] ensure: [f close]
! !


!TimeZoneProxy methodsFor: 'initialize-release' stamp: 'dtl 11/14/2004 12:47'!
initialize

	self name; abbreviation.
	self transform addDependent: self.
	TimeZoneDatabase addDependent: self
! !

!TimeZoneProxy methodsFor: 'accessing' stamp: 'dtl 11/14/2004 13:17'!
abbreviation
	"Answer the value of abbreviation"

	^ abbreviation ifNil: [abbreviation _ self transform abbreviation]! !

!TimeZoneProxy methodsFor: 'accessing' stamp: 'dtl 11/14/2004 13:14'!
abbreviation: newAbbrev
	"Set the value of abbreviation"

	^ self
		abbreviation: newAbbrev
		ifNotValid: [self notify: '"', newAbbrev asString, '" is not a valid abbreviation for this time zone']
! !

!TimeZoneProxy methodsFor: 'accessing' stamp: 'dtl 11/14/2004 12:30'!
abbreviation: anObject ifNotValid: errorBlock
	"Set the value of abbreviation. If the abbreviation is not valid for the current
	time zone, evaluate errorBlock."

	| newAbbrev possibleValues |
	newAbbrev _ anObject asString.
	possibleValues _ (self transform isNil
			ifTrue: [Array with: newAbbrev]
			ifFalse: [transform tzNameAbbreviations])
		collect: [:e | e asString].
	(possibleValues includes: newAbbrev)
		ifTrue: [abbreviation _ newAbbrev]
		ifFalse: [errorBlock value]
! !

!TimeZoneProxy methodsFor: 'accessing' stamp: 'dtl 11/14/2004 07:33'!
name

	^ transform isNil
		ifTrue: [transform name]
		ifFalse: [super name]! !

!TimeZoneProxy methodsFor: 'accessing' stamp: 'dtl 9/13/2004 12:31'!
transform
	"Answer the value of transform"

	^ transform! !

!TimeZoneProxy methodsFor: 'accessing' stamp: 'dtl 9/13/2004 12:31'!
transform: anObject
	"Set the value of transform"

	transform _ anObject! !

!TimeZoneProxy methodsFor: 'time zone offset' stamp: 'dtl 11/13/2004 12:20'!
dstTransitionTimes

	^ self transform dstTransitionTimes! !

!TimeZoneProxy methodsFor: 'time zone offset' stamp: 'dtl 11/23/2004 12:05'!
localOffsetSecondsAt: aPointInTime

	^ self transform localOffsetSecondsAt: aPointInTime! !

!TimeZoneProxy methodsFor: 'time zone offset' stamp: 'dtl 11/11/2004 20:01'!
offset

	^ Duration seconds: (self transform localOffsetSecondsAt: PointInTime now)! !

!TimeZoneProxy methodsFor: 'leap seconds' stamp: 'dtl 11/7/2004 20:42'!
leapSecondsAt: aPointInTime

	^ self transform leapSecondsAt: aPointInTime! !

!TimeZoneProxy methodsFor: 'printing' stamp: 'dtl 11/14/2004 13:17'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: ' for ', self transform name, ' (', self abbreviation asString, ')'
! !

!TimeZoneProxy methodsFor: 'updating' stamp: 'dtl 11/14/2004 13:14'!
update: aParameter 
	"If a new system time zone database has been created, update my reference to it.
	If the default transform has changed, update my abbreviation if necessary."

	aParameter == #timeZoneDatabaseForSystem
		ifTrue: [(self transform isKindOf: TimeZoneDatabase)
				ifTrue:
					[self transform: TimeZoneDatabase systemDatabase.
					transform addDependent: self.
					self abbreviation: self abbreviation
						ifNotValid: [self abbreviation: self transform abbreviation]]].
	aParameter == #timeZoneDatabaseDefaultZone
		ifTrue: [self
			abbreviation: self abbreviation
			ifNotValid: [self abbreviation: self transform abbreviation ifNotValid: []]]! !


!TimeZoneProxy class methodsFor: 'instance creation' stamp: 'dtl 11/10/2004 05:24'!
dynamicTransform
	"Answer a proxy that uses the system time zone database. This in turn
	will refer to the currently active local time transform in the database."

	"TimeZoneProxy dynamicTransform"

	^ (self basicNew transform: TimeZoneDatabase systemDatabase) initialize! !

!TimeZoneProxy class methodsFor: 'set local time zone' stamp: 'dtl 11/10/2004 05:22'!
setDynamicTransform
	"Set the proxy to use the system time zone database. When the default time
	zone for the database is changed, the change is automatically reflected in
	this proxy. The time zone abbreviation for the active time zone is selected
	from one of possibly many valid abbreviations for that time zone, and will
	not in general be an expected value.

	This configuration should be used in applications where the system time
	zone may be reset in the time zone database, and where the time zone
	abbreviation is relatively unimportant."

	"TimeZoneProxy setDynamicTransform"

	| proxy |
	proxy _ self dynamicTransform.
	DateAndTime localTimeZone: proxy.
	^ proxy! !

!TimeZoneProxy class methodsFor: 'set local time zone' stamp: 'dtl 11/14/2004 13:06'!
setDynamicTransform: timeZoneName
	"Set the proxy to use the system time zone database, and set the database
	to use timeZoneName as the default time zone."

	"TimeZoneProxy setDynamicTransform: '/right/America/Detroit'"

	| proxy |
	proxy _ self setDynamicTransform.
	self setLocalTimeZone: timeZoneName.
	^ proxy
! !

!TimeZoneProxy class methodsFor: 'set local time zone' stamp: 'dtl 11/14/2004 13:09'!
setDynamicTransform: timeZoneName abbreviation: abbreviation
	"Set the proxy to use the system time zone database, and set the database
	to use timeZoneName as the default time zone. Set the time zone abbreviation
	to one of the possibly several valid abbreviations for timeZoneName."

	"TimeZoneProxy setDynamicTransform: '/right/America/Detroit' abbreviation: 'EST'"

	^ (self setDynamicTransform: timeZoneName) abbreviation: abbreviation
! !

!TimeZoneProxy class methodsFor: 'set local time zone' stamp: 'dtl 11/14/2004 12:58'!
setLocalTimeZone: tzName
	"To find a time zone for Detroit:
		TimeZoneDatabase grepFor: 'Detroit'
	Then use this method to set the desired time zone."

	"TimeZoneProxy setLocalTimeZone: 'Europe/Moscow' "
	"TimeZoneProxy setLocalTimeZone: 'right/America/Detroit' "
	"TimeZoneProxy setLocalTimeZone: 'posix/America/Detroit' "
	"TimeZoneProxy setLocalTimeZone: 'America/Detroit' "
	"TimeZoneProxy setLocalTimeZone: 'UTC' "
	"TimeZoneProxy setLocalTimeZone: 'GMT' "
	"TimeZoneProxy setLocalTimeZone: 'Europe/London' "
	"TimeZoneProxy setLocalTimeZone: 'Africa/Johannesburg' "
	"TimeZoneProxy setLocalTimeZone: 'America/Los_Angeles' "

	TimeZoneDatabase setLocalTimeZone: tzName
! !

!TimeZoneProxy class methodsFor: 'private' stamp: 'dtl 11/8/2004 21:13'!
setDynamicLocalTransform: aTimeZoneRuleSet abbreviation: abbreviation
	"Set the local time zone to a proxy for the TimeZoneRuleSet that describes this
	time zone. The time zone offset value will be dynamically calculated for any
	point in time. This causes the time zone offset to be dynamically corrected for
	daylight savings time and other time zone anomolies.

	Using a time zone proxy causes methods such as DateAndTime class>>now and
	Time class>>now to run more slowly, which could be a concern in applications
	that do a very large number of system time queries."

	"TimeZoneProxy setDynamicLocalTransform:
			(TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit')
		abbreviation: 'EST'"

	| proxy |
	proxy _ self transform: aTimeZoneRuleSet abbreviation: abbreviation.
	DateAndTime localTimeZone: proxy.
	^ proxy! !

!TimeZoneProxy class methodsFor: 'private' stamp: 'dtl 9/20/2004 19:40'!
setStaticLocalTransform: aTimeZoneRuleSet abbreviation: abbreviation
	"Set the local time zone to a TimeZone with an offset corresponding to the
	offset of the local time zone at the present time. Time zone offsets will not
	be dynamically calculated, and this TimeZone will not be valid across
	daylight savings time changes.
	
	Unless the speed of system time queries is a serious concern, the
	#setDynamicLocalTransform:abbreviation: method is preferred."

	"TimeZoneProxy setStaticLocalTransform: TimeZoneDatabase systemDatabase defaultTimeZone abbreviation: 'EST'"

	| proxy tz |
	proxy _ self transform: aTimeZoneRuleSet abbreviation: abbreviation.
	tz _ TimeZone
		offset: proxy offset
		name: proxy name
		abbreviation: proxy abbreviation.
	DateAndTime localTimeZone: tz.
	^ proxy! !

!TimeZoneProxy class methodsFor: 'private' stamp: 'dtl 11/10/2004 05:23'!
transform: aTimeZoneRuleSet
	"Answer a proxy that uses aTimeZoneRuleSet. The time zone abbreviation will be
	the first of possibly many valid abbreviations provided in the time zone
	rules."

	"TimeZoneProxy transform: TimeZoneDatabase systemDatabase defaultTimeZone"

	^ (self basicNew transform: aTimeZoneRuleSet) initialize! !

!TimeZoneProxy class methodsFor: 'private' stamp: 'dtl 9/13/2004 12:59'!
transform: aTimeZoneRuleSet abbreviation: abbreviation
	"A time zone rule set typically has more than one abbreviation. This allows a
	specific abbreviation to be used. The abbreviation must be one of the abbreviations
	listed in the rule set."

	"TimeZoneProxy transform: TimeZoneDatabase systemDatabase defaultTimeZone abbreviation: 'EST'"
	"TimeZoneProxy transform: TimeZoneDatabase systemDatabase defaultTimeZone abbreviation: 'XXX'"

	(aTimeZoneRuleSet tzNameAbbreviations includes: abbreviation asSymbol)
		ifTrue:
			[^ (self basicNew
				transform: aTimeZoneRuleSet;
				abbreviation: abbreviation asString) initialize]
		ifFalse:
			[self notify: abbreviation, ' not valid, must be one of ',
				aTimeZoneRuleSet tzNameAbbreviations asString]
! !


!TimeZoneRule methodsFor: 'printing' stamp: 'dtl 11/20/2004 12:47'!
printOn: aStream

	"^ self printSimplyOn: aStream"
	^ self printVerboseOn: aStream
! !

!TimeZoneRule methodsFor: 'printing' stamp: 'dtl 11/20/2004 12:16'!
printVerboseOn: aStream
	"Print enough detail to allow the output to be compared with the output of
	the zdump test utility, as found on may Unix-like systems."

	(PointInTime fromAbsoluteSeconds: self transitionTime) asDateAndTime printOn: aStream.
	aStream nextPutAll: (' ', self abbreviation, ' (').
	transitionTime printOn: aStream.
	aStream nextPutAll: ', '.
	offsetSeconds printOn: aStream.
	aStream nextPutAll: ', '.
	isDstFlag printOn: aStream.
	aStream nextPutAll: ', '.
	abbreviation printOn: aStream.
	aStream nextPutAll: ', '.
	isStdTimeTransition printOn: aStream.
	aStream nextPutAll: ', '.
	isUtcTransition printOn: aStream.
	aStream nextPut: $)! !

!TimeZoneRule methodsFor: 'printing' stamp: 'dtl 12/28/1999 20:01'!
printVerboseOn: aStream withLeapSeconds: anInteger
	"Print enough detail to allow the output to be compared with the output of
	the zdump test utility, as found on may Unix-like systems. This rule does
	not know about leap seconds, which are maintained in a separate rule table,
	so any leap second compensation must be provided as a parameter."

	| dt dtUtc leap timeWithLeapOffset |
	anInteger isNil ifTrue: [leap _ 0] ifFalse: [leap _ anInteger].
	timeWithLeapOffset _ self transitionTime - leap.
	dt _ (PointInTime fromPosixSeconds: timeWithLeapOffset) asUtcDateAndTime.
	dtUtc _ (PointInTime fromPosixSeconds: (timeWithLeapOffset + self offsetSeconds)) asUtcDateAndTime.
	(dt at: 1) printOn: aStream.
	aStream nextPut: $ .
	(dt at: 2) printOn: aStream.
	aStream nextPutAll: ' UTC = '.
	(dtUtc at: 1) printOn: aStream.
	aStream nextPut: $ .
	(dtUtc at: 2) printOn: aStream.
	aStream nextPutAll: (' ', self abbreviation, ' (').
	transitionTime printOn: aStream.
	aStream nextPutAll: ', '.
	offsetSeconds printOn: aStream.
	aStream nextPutAll: ', '.
	isDstFlag printOn: aStream.
	aStream nextPutAll: ', '.
	abbreviation printOn: aStream.
	aStream nextPutAll: ', '.
	isStdTimeTransition printOn: aStream.
	aStream nextPutAll: ', '.
	isUtcTransition printOn: aStream.
	aStream nextPut: $)! !


!TimeZoneRuleSet methodsFor: 'accessing' stamp: 'dtl 11/14/2004 07:39'!
abbreviation
	"Answer one of possibly several abbreviations for this time zone"

	^ self tzNameAbbreviations anyOne asString! !

!TimeZoneRuleSet methodsFor: 'accessing' stamp: 'dtl 11/13/2004 12:09'!
dstTransitionTimes

	^ self transitionTimeTable
		collect: [:rule | PointInTime fromAbsoluteSeconds: rule transitionTime]! !

!TimeZoneRuleSet methodsFor: 'accessing' stamp: 'dtl 11/13/2004 12:15'!
leapSecondTransitionTimes

	self leapSecondRuleSet ifNil: [^ #()].
	^ (self leapSecondRuleSet leapSecondTable
		collect: [:r | PointInTime fromAbsoluteSeconds: r first]) asArray! !

!TimeZoneRuleSet methodsFor: 'accessing' stamp: 'dtl 11/13/2004 22:59'!
name
	"Name when used as a TimeZone"

	^ (self timeZoneName ifNil: [super name]) asString! !

!TimeZoneRuleSet methodsFor: 'private' stamp: 'dtl 10/3/2004 15:53'!
zoneRuleFor: posixSeconds
	"Answer the zone data record for a point in time (an Integer). Assuming
	that elements of transitionTimeTable are sorted by transitionTime, this is the last
	such element for which transitionTime is less than posixSeconds."

	| recordsBeforeThisTime |
	transitionTimeTable isNil ifTrue: [^ nil].
	transitionTimeTable isEmpty ifTrue: [^ nil].
	recordsBeforeThisTime _ transitionTimeTable select: [:e | e transitionTime <= posixSeconds].
	recordsBeforeThisTime isEmpty
		ifTrue: [^ transitionTimeTable first]		"Default to first rule in the list"
		ifFalse: [^ recordsBeforeThisTime last]! !


!WallClock methodsFor: 'updating' stamp: 'dtl 11/12/2004 17:00'!
update: aParameter

	| dt |
	now isNil ifFalse: [
		dt _ self now asDateAndTimeArrayForTimeZone: self timeZone.
		date _ dt at: 1.
		time _ dt at: 2]
! !


!WallClock class methodsFor: 'instance creation' stamp: 'dtl 11/12/2004 17:32'!
for: aLocalTimeTransformName
	"Answer a WallClock in the specified time zone. Note that names beginning
	with 'right' include leap second tables, while names beginning with 'posix'
	do not. Select the type of time zone that matches the time setting for your
	computer's operating system."

	"WallClock for: 'right/America/Detroit'"
	"WallClock for: 'posix/America/Detroit'"
	"WallClock for: 'right/Europe/Berlin'"
	"WallClock for: 'right/Europe/Moscow'"

	| transform |
	transform _ TimeZoneDatabase systemDatabase timeZoneFor: aLocalTimeTransformName.
	^ transform isNil
		ifTrue: [nil]
		ifFalse: [self forTimeZone: transform]
! !


!TimeZoneProxy class reorganize!
('instance creation' dynamicTransform)
('set local time zone' setDynamicTransform setDynamicTransform: setDynamicTransform:abbreviation: setLocalTimeZone:)
('private' setDynamicLocalTransform:abbreviation: setStaticLocalTransform:abbreviation: transform: transform:abbreviation:)
!

TimeZoneProxy removeSelector: #name:!

!TimeZoneProxy reorganize!
('initialize-release' initialize)
('accessing' abbreviation abbreviation: abbreviation:ifNotValid: name transform transform:)
('time zone offset' dstTransitionTimes localOffsetSecondsAt: offset)
('leap seconds' leapSecondsAt:)
('printing' printOn:)
('updating' update:)
!

TimeZoneDstTransitionWatcher initialize!

!TimeZoneDstTransitionWatcher class reorganize!
('class initialization' initialize startMonitoring stopMonitoring)
('instance creation' clockWatcher)
('tests' verifyDstTransition)
!

TimeZoneDstTransitionWatcher removeSelector: #updateLastTick!

!TimeZoneDstTransitionWatcher reorganize!
('initialize-release' initialize start stop)
('accessing' accessProtect lastTick lastTick: monitorProcess monitorProcess: transitionTick transitionTick:)
('testing' fallBackOccurredDuringHourBefore:)
('monitor' isFallBack: monitorProcessBlock updateLastTick:)
('updating' lastDstTransitionBefore: probableLastTransitionTick probableLastTransitionTickBefore: update:)
!


!TimeZoneDatabase class reorganize!
('instance creation' buildSystemDatabase buildSystemDatabaseForSqueak buildSystemDatabaseFromExamples fromExampleFiles fromFiles:prefixPath: fromFiles:prefixPath:pathSeparator: fromZoneinfoDirectory: systemDatabase)
('setting local time zone' grepFor: setLocalTimeZone:)
('initialize-release' initialize thisSystemDatabase:)
('examples' convertDate:time:fromTimeZone:toTimeZone: partyTimes partyTimesForNewYear: sortedArrayOfWorldTimesAtPointInTime: worldTimes worldTimesAtPointInTime: worldTimesForDate:time:inTimeZone:)
('defaulting' defaultLocation exampleTzFiles indexSeparator tzExamplesPrefixPath tzPrefixPath)
('database verification' dumpRulesToFilesIn:)
('fileIn/Out' fileOutClasses)
('private' changeSeparatorFrom:to:in: dateFromString: directoryContents: directoryFilesIn:inBasePath: isTzFileType: pathFromPrefix:andName: platform separator showProgressFrom:to:withMessage:forBlock: supportsProgressBar tzFilesInDirectory:inBasePath: tzFilesInDirectoryTree:)
!


!TimeZoneDatabase reorganize!
('accessing' defaultLocation defaultLocation: defaultTimeZone indexSeparator indexSeparator: leapSecondRuleSet leapSecondRuleSet: timeZones)
('adding' indexAt:put: indexAt:withSeparator:put: loadDataFrom: loadDataFrom:prefixPath: loadDataFrom:prefixPath:pathSeparator:condenseDuplicates: updateLeapSecondRulesFrom:)
('querying' allTimeZones grepFor: hoursFrom:to: leapSecondsFor:at: offsetFor:at: offsetFrom:to: timeZoneFor: timeZoneFor:pathSeparator: timeZoneName timeZoneNames tzNameAbbreviations)
('offsets from UTC' localOffsetSecondsAt:)
('leap seconds' leapSecondsAt:)
('time zone proxy' abbreviation dstTransitionTimes name offset)
('*kernel-Chronology' durationFrom:to:)
('normalizing' combineDuplicateTables: normalize setLeapSecondRulesFrom:)
('private' addZone:toHierarchy:withKeys: addZonesTo:from: isContainerNode: isTerminalLeaf: leapSecondsRuleFor: timeZoneIndexedBy:inHierarchy: tokensFor:withSeparator:)
!


!TimeZone reorganize!
('accessing' abbreviation abbreviation: name name: offset offset:)
('private' printOn:)
('*time-TimeZones' dstTransitionTimes leapSecondsAt: localOffsetSecondsAt:)
!

TimeTransformTest removeSelector: #setDstTransitionWatcher!
TimeTransformTest removeSelector: #setDstTransitionWatcherFall2400!
TimeTransformTest removeSelector: #setDstTransitionWatcherSpring2400!

!TimeTransformTest reorganize!
('documentation' squeakSecondsClockInfo utcClockInfo)
('running' setUp tearDown)
('testing-TZ offset at DST transition' testDaylightSavingsTimeOffset testDaylightSavingsTimeOffsetAtTransitionTime testDaylightSavingsTimeOffsetImmediatelyAfter testDaylightSavingsTimeOffsetImmediatelyBefore testDaylightSavingsTimeOffsetLongAfter testDaylightSavingsTimeOffsetLongBefore)
('testing-PointInTime seconds' testAbsoluteTimeToDateAndTime testAbsoluteTimeToDateAndTimeSeconds testAbsoluteTimeToLocalSmalltalkSeconds testConversionToLocalSmalltalkSeconds testConversionToLocalSmalltalkSeconds2 testConversionToLocalSmalltalkSecondsNoLeapSeconds testConvertLocalSecondsToPointInTime testDateAndTimeSummer2004EST testLeapSecondCount testPointInTimeNowSeconds testSmalltalkEpoch)
('testing-PointInTime TZ offset' testOffsetAsUtc testOffsetAsUtcWithLeapSeconds testPointInTimeAtDstTransition testPointInTimeAtDstTransitionNoLeapSeconds testPointInTimeImmediatelyAfter testPointInTimeImmediatelyAfterNoLeapSeconds testPointInTimeImmediatelyBefore testPointInTimeImmediatelyBeforeNoLeapSeconds testPointInTimeOneDayAfter testPointInTimeOneDayBefore testPointInTimeOneHalfHourAfter testPointInTimeOneHourAfter testPointInTimeOneHourBefore)
('testing-Posix seconds' testPointInTimePosixOffset testPosixSecondsVersusAtomicSeconds testPosixToPointInTimeConversion testPosixToPointInTimeConversionNoLeapSeconds)
('testing-DateAndTime' testDateAndTimeAtDstTransition testDateAndTimeAtDstTransitionNoLeapSeconds testDateAndTimeConversion testDateAndTimeImmediatelyAfter testDateAndTimeImmediatelyAfterNoLeapSeconds testDateAndTimeImmediatelyBefore testDateAndTimeImmediatelyBeforeNoLeapSeconds testDateAndTimeNowSeconds testDateAndTimeOffsetDuration testDateAndTimeOneDayAfter testDateAndTimeOneDayBefore testDateAndTimeOneHalfHourAfter testDateAndTimeOneHourAfter testDateAndTimeOneHourBefore testDateAndTimeThreeHoursAfter testDateAndTimeTwoHoursAfter testDateAndTimeTwoHoursBefore testDateFromYearMonthDay)
('testing-Date and Time array' testCreateDateAndTimeArray testTransformADateAndATimeFromTimeZone testTransformADateAndATimeToTimeZone)
('testing-local conversion' testAmbiguousConversionAfterDst testAmbiguousConversionForLeapSeconds testAmbiguousConversionToLocalSeconds)
('testing-Date' testDateAtDstTransition testDateAtDstTransitionNoLeapSeconds testDateFirstSecondOfDayOfDstTransition testDateFirstSecondOfDayOfDstTransitionNoLeapSeconds testDateInDstPeriods testDateLastSecondOfDayBeforeDstTransition testDateLastSecondOfDayBeforeDstTransitionNoLeapSeconds)
('testing-Time' testAsTime testDateAndTimeFromSeconds testDateAndTimeFromSecondsNoLeapSeconds testTimeAsDateAndTime testTimeNow testTimeNowSeconds)
('testing-Duration' testDurationForSamePointInTimeIsZero testDurationWithNoTransition testTwoDateAndTimesTheSame testTwoDateAndTimesWithDifferentOffset)
('testing-Duration DST' testDurationAcrossDstTransition testSecondsCountAcrossDstTransition testSecondsCountAcrossDstTransitionNoLeapSeconds)
('testing-Duration leap seconds' testSecondsCountAcrossLeapSecondTransition)
('testing-arithmetic' testAddDurationAcrossDstTransition testAddDurationAcrossLeapSecondTransition testSubtractDateAndTime testSubtractDateAndTimeAcrossDstTransition testSubtractDurationAcrossDstTransition testSubtractDurationAcrossLeapSecondTransition)
('testing-time plugin' testPrimPosixTimeMicrosecondResolution testPrimPosixTimeMicrosecondResolutionOffset testPrimSecondsClock testPrimVersusApproximateTimeNow)
('testing-misc' testLocalDetroitEpoch testPosixEpoch testTimeZoneTransitionSecond)
('private' absoluteDstTransition localSmalltalkDstTransition posixDstTransition posixOffset primPosixTimeAndLocalOffset primSecondsClock setDstTransitionWatcherFall2004 setDstTransitionWatcherSpring2004)
!

PointInTime class removeSelector: #allAbsoluteTimesCorrespondingTo:!
PointInTime class removeSelector: #anyAbsoluteSecondsAtSmalltalkSeconds:inTimeZone:!
PointInTime class removeSelector: #approximateTimeNow!
PointInTime class removeSelector: #getAbsoluteSeconds!
PointInTime class removeSelector: #getAbsoluteSecondsFor:!
PointInTime class removeSelector: #getAbsoluteSecondsForSqueak!
PointInTime class removeSelector: #getLocalSmalltalkSeconds!
PointInTime class removeSelector: #getPosixSeconds!
PointInTime initialize!
PointInTime class removeSelector: #posixSecondsAtAbsoluteSeconds:!
PointInTime class removeSelector: #posixSecondsAtAbsoluteSeconds:inTimeZone:!
PointInTime class removeSelector: #posixSecondsNow!
PointInTime class removeSelector: #probableAbsoluteSecondsAtSmalltalkSeconds:inTimeZone:!
PointInTime class removeSelector: #probablePointInTimeAtSmalltalkSeconds:inTimeZone:!

!PointInTime class reorganize!
('documentation' squeakSecondsClockInfo utcClockInfo)
('initialize-release' initialize numericRepresentationSelector:)
('instance creation' date:time: date:time:timeZone: fromAbsoluteSeconds: fromPosixSeconds: fromPosixSeconds:inTimeZone: fromSeconds: fromSmalltalkSeconds: fromSmalltalkSeconds:inTimeZone: fromUtcSmalltalkSeconds: now)
('general inquiries' absoluteSecondsAtPosixSeconds:inTimeZone: absoluteSecondsAtSmalltalkSeconds:inTimeZone: absoluteTimeAtSmalltalkSeconds:inTimeZone: allAbsoluteSecondsCorrespondingTo:inTimeZone: allAbsoluteTimesCorrespondingTo:inTimeZone: dateAndTimeArrayFromSmalltalkSeconds: dateAndTimeFromSmalltalkSeconds: dateAndTimeNow dateFromSmalltalkSeconds: firstAbsoluteSecondsAtSmalltalkSeconds:inTimeZone: posixSecondsAtSmalltalkSeconds: posixSecondsAtSmalltalkSeconds:inTimeZone: tzOffsetAtSmalltalkSeconds:inTimeZone:)
('private' platform platformSymbol posixOffset testFromSmalltalkSeconds testLocalSeconds)
('private-system clock' absoluteSecondsNow getAbsoluteSecondsForStx getAbsoluteSecondsFromLocalSecondsClock getAbsoluteSecondsFromPluginOrLocalSecondsClock getStxSeconds)
('primitives' primLocalSecondsClockWithOffset primPosixTime primPosixTimeAndLocalOffset primPosixTimeMicrosecondResolution)
!

PointInTime removeSelector: #approximateTimeNow!
PointInTime removeSelector: #getAbsoluteSecondsFor:!
PointInTime removeSelector: #getPosixSeconds!
PointInTime removeSelector: #getStxSeconds!
PointInTime removeSelector: #primLocalSecondsClockWithOffset!
PointInTime removeSelector: #primPosixTime!
PointInTime removeSelector: #primPosixTimeAndLocalOffset!
PointInTime removeSelector: #primPosixTimeMicrosecondResolution!
PointInTime removeSelector: #setAbsoluteTime!

!PointInTime reorganize!
('accessing' absoluteTime absoluteTime: currentLeapSeconds currentOffsetSeconds)
('converting' asAbsoluteSeconds asDateAndTime asDateAndTimeArrayForTimeZone: asDateAndTimeForTimeZone: asFloat asInteger asLocalDateAndTime asLocalSmalltalkSeconds asNumber asPosixSeconds asPosixSecondsForTimeZone: asSmalltalkSecondsForTimeZone: asUtcDateAndTime asUtcSmalltalkSeconds)
('comparing' < = hash)
('arithmetic' -)
('printing' printOn:)
!

DateAndTime class removeSelector: #posixOffset!

!DateAndTime class reorganize!
('ansi protocol' clockPrecision now year:day:hour:minute:second: year:day:hour:minute:second:offset: year:month:day:hour:minute:second: year:month:day:hour:minute:second:offset:)
('squeak protocol' current date:time: epoch fromString: julianDayNumber: localOffset localTimeZone localTimeZone: midnight new noon readFrom: today tomorrow year:day: year:month:day: year:month:day:hour:minute: year:month:day:hour:minute:second:nanoSecond:offset: yesterday)
('smalltalk-80' fromSeconds: millisecondClockValue totalSeconds)
('*time-TimeZones' fromSeconds:offsetSeconds:)
!


!DateAndTime reorganize!
('ansi protocol' + - < = asLocal asUTC dayOfMonth dayOfWeek dayOfWeekAbbreviation dayOfWeekName dayOfYear hash hour hour12 hour24 isLeapYear meridianAbbreviation minute month monthAbbreviation monthName offset offset: second timeZoneAbbreviation timeZoneName year)
('squeak protocol' asDate asDateAndTime asDuration asMonth asNanoSeconds asTime asTimeStamp asWeek asYear dayMonthYearDo: duration julianDayNumber middleOf: midnight nanoSecond noon printHMSOn: printOn: printOn:withLeadingSpace: printYMDOn: printYMDOn:withLeadingSpace: to: to:by: to:by:do: utcOffset:)
('smalltalk-80' asSeconds day daysInMonth daysInYear daysLeftInYear firstDayOfMonth hours minutes monthIndex seconds)
('*time-UTC' asPointInTime asPointInTimeInTimeZone:)
('private' ticks ticks:offset:)
!

